@echo on
REM	* UDM_rel.bat <VSS_DB_DIR> <VSS_User> <UDM_Download_Dir> 
REM	*		<3rdParty_Dir> <3rdParty_Download_Dir> <UDM_Ver> <GReAT_Dir> [-b] [-p]
REM	*
REM	* This batch file creates an UDM release
REM	*	- get latest source files from a VSS database
REM	*	- get 3rd party files from a specific location
REM	*	- update the version numbers in binaries and readme files
REM	*	- build the binary files
REM	*	- build the release
REM	*
REM	* Arguments:
REM	*	1. <VSS_DB_DIR>: The location of the SRCSAFE.INI file for the UDM VSS database. 
REM	*	2. <VSS_User>: VSS user name to connect to the UDM VSS database
REM	*	3. <UDM_Download_Dir>: The directory where UDM source files should go to
REM	*	4. <3rdParty_Dir>: The location of the UDM 3rdParty zip files: 3rdParty_source.zip, 3rdParty_bin.zip
REM	*	5. <3rdParty_Bin_Download_Dir>: The directory where 3rdParty_bin.zip will be extracted. 
REM	*		This argument is ignored if the -b switch is specified. (but the user still has to specify this argument)
REM	*	6. <3rdParty_Source_Download_Dir>: The directory where 3rdParty_source.zip will be extracted. 
REM	*		This argument is ignored if the -b switch is specified. (but the user still has to specify this argument)
REM	*		(3rdParty_source.zip is always extracted to %UDM_PATH%\3rdParty)
REM	*	7. <GReAT_Dir>: The directory where GReAT is either installed or being developed. 
REM	*		There should be a bin directory under this directory with MetaGME2UML.dll in it.
REM	*	8. <UDM_Ver>: Version number of the UDM release
REM	*
REM	*  Switches: (specify after arguments!)
REM	*	[-b]: Optional. Fetch&Build mode (don't build release)
REM	* 	[-p]: Optional. Do not download 3rdparty sources
REM	*		
REM	* Script files:
REM	*	-
REM	*
REM	* Requirements:
REM	*	- Microsoft Visual Studio 6.0 with SP6
REM	*	- Microsoft Visual SourceSafe 6.0
REM	*	- InstallShield Developer 8.0 with SP1
REM	*	- GME 4.5.18
REM	*	- Java Runtime Environment
REM	*	- Cint C/C++ interpreter (http://root.cern.ch/root/Cint.html)
REM	*		(Please note you need to follow the cint source distribution installation 
REM	*		 instructions, which includes running setupxxx.bat)
REM	*
REM	* Missing features:
REM	*	1. updating CHANGELOG.txt, readme.txt, releaseinfo.txt
REM	*	2. versioning of the binaries


REM	* Initialization
	set UDM_VER=%8
	set GREAT_PATH=%7
	set UDM_PATH=%3
	set UDM_LOGF=%UDM_PATH%\Release\udm_rel_log.txt
	echo ******************************************************************************** >  %UDM_LOGF%
	echo * UDM release build process started %DATE% %TIME% >> %UDM_LOGF%
	echo ******************************************************************************** >>  %UDM_LOGF%
REM 	* Get Sources from VSS
	echo * SS.exe invoked %DATE% %TIME% >> %UDM_LOGF%
	call %UDM_PATH%\Release\fetch_sources.bat %1 %2 $/MobiesTransition/Udm %3
	IF "%ERRORLEV%" == "1" echo * SS.exe returned 1. This command ran successfully, but your next VSS command may fail. %DATE% %TIME% >> %UDM_LOGF%
	IF "%ERRORLEV%" == "2" GOTO ErrorHandler
REM	* Unzip 3rd party libraries
	if  "%9" == "-p" GOTO Skip3rdParty
	if  "%10" == "-p" GOTO Skip3rdParty
	%UDM_PATH%\Release\unzip.exe -u -q -o %4\3rdparty_source.zip -d %6
REM	* if fetch&build do not extract 3rdParty_bin.zip
	if  "%9" == "-b" GOTO Skip3rdParty
	if  "%10" == "-b" GOTO Skip3rdParty
	%UDM_PATH%\Release\unzip.exe -u -q--o %4\3rdparty_bin.zip -d %5
	set UDM_3RDPARTY_BIN=%5
:Skip3rdParty
	set UDM_3RDPARTY=%6
REM	* Update the version number in UdmBase.h ???
REM	* Build the binaries
	echo ******************************************************************************** >>  %UDM_LOGF%
	echo * MSDEV.exe invoked %DATE% %TIME% >>  %UDM_LOGF%
	call %UDM_PATH%\Release\build_bin.bat
	IF "%ERRORLEV%" == "1" GOTO ErrorHandler
REM	* Fetch and Build
	if  "%9" == "-b" GOTO BuildOnlyHandler
	if  "%10" == "-b" GOTO BuildOnlyHandler
REM	* Do the versioning of the release, add changelog
	echo ******************************************************************************** >>  %UDM_LOGF%
	echo * is_ver.vbs invoked %DATE% %TIME% >>  %UDM_LOGF%
	cd /d %UDM_PATH%\InstallShield
	ss.exe Checkout $/MobiesTransition/Udm/InstallShield/UDM.ism -I
	cd /d %UDM_PATH%\InstallShield\UDM\
	ss.exe Checkout $/MobiesTransition/Udm/InstallShield/UDM/setup.obs -I
	REM
	call %UDM_PATH%\Release\is_ver.vbs "%UDM_PATH%\InstallShield\UDM.ism" %UDM_VER%
	REM  ver.txt, readme.txt
	REM ...
	REM changelog
	REM ...
REM	* Build the release
	echo ******************************************************************************** >>  %UDM_LOGF%
	echo * is_path.vbs invoked %DATE% %TIME% >>  %UDM_LOGF%
	call %UDM_PATH%\Release\is_path.vbs "%UDM_PATH%\InstallShield\UDM.ism" "%UDM_PATH%" "%UDM_3RDPARTY_BIN%" "%GREAT_PATH%"
	echo ******************************************************************************** >>  %UDM_LOGF%
	echo * is_build.bat invoked %DATE% %TIME% >>  %UDM_LOGF%
	call %UDM_PATH%\Release\is_build.bat "%UDM_PATH%\InstallShield\UDM.ism" "%UDM_PATH%\InstallShield\UDM\setup.rul"
REM	* Finished
	GOTO End

:ErrorHandler
	echo ******************************************************************************** >>  %UDM_LOGF%
	echo * udm_rel.bat FAILED at %DATE% %TIME% >>  %UDM_LOGF%
	GOTO End
	
:BuildOnlyHandler
	echo ******************************************************************************** >>  %UDM_LOGF%
	echo * udm_rel.bat Fetch and Build mode finished at %DATE% %TIME% >>  %UDM_LOGF%
	
:End
	echo ******************************************************************************** >>  %UDM_LOGF%
	echo * UDM release build process ended %DATE% %TIME% >> %UDM_LOGF%
