/*
Copyright (c) Vanderbilt University, 2000-2001
ALL RIGHTS RESERVED
Vanderbilt University disclaims all warranties with regard to this
software, including all implied warranties of merchantability
and fitness.  In no event shall Vanderbilt University be liable for
any special, indirect or consequential damages or any damages
whatsoever resulting from loss of use, data or profits, whether
in an action of contract, negligence or other tortious action,
arising out of or in connection with the use or performance of
this software.

  FILE:		UdmCInt.h
  SYNOPSIS:	CInt interface classes - common classes shared by Udm and CInt
  CREATED:	08/10/02
  AUTHOR:	Endre Magyari(endre.magyari@vanderbilt.edu)

  CHANGELOG:
		04/27/04 -	endre
			added
			- copy constructor
			- += int, += char operators
		
		10/12/02	operator == for cint_string added, requested by Feng
		09/30/02	operator += for cint_string added, requested by Feng


*/

class UdmPseudoObject;

class UdmPseudoObjectS
{
	UdmPseudoObject * impl;
	int length;
public:

	UdmPseudoObjectS(int length);
	UdmPseudoObjectS(const UdmPseudoObjectS &frm);
	UdmPseudoObjectS& operator=(const UdmPseudoObjectS &frm);
	bool operator !() const;
	operator bool()  const;
	bool overflow;

	
	//for getting object no index
	UdmPseudoObject& operator[](const int index) const;
	//for setting object no index
	void SetAt(const int index, const UdmPseudoObject& item);
	int GetLength() const;

	~UdmPseudoObjectS();
};

#ifdef __MAKECINT__
#define ADD_CINT_STRING
#else
#ifdef CS_IN_UDMCINT
#define ADD_CINT_STRING
#endif
#endif

#ifdef ADD_CINT_STRING
class cint_string
{
	char * impl;
	int length;
	
public:
	cint_string();										//for creating a NULL string
	cint_string(int length);							//for creating an empty string, length chars long

	cint_string(const char * frm);						//for creating a cint_string containing frm
	cint_string(const cint_string& frm);						
	cint_string& operator=(const cint_string& frm);		
	cint_string& operator+=(const cint_string& what);	//requested by Feng
	cint_string& operator+=(const char what);	//requested by vijan
	cint_string& operator+=(const int what);	//requested by vijan

	bool operator ==(const cint_string& what);			//also requested by Feng

	bool CopyFrom(const char *);
	bool operator !() const;
	operator bool() const;

	
	~cint_string();
	int comp(const char * compare_to) const;
	bool overflow;
	char * buffer() const;
};
#endif

struct AssociationInfo
{

	cint_string assoc_class;//can be NULL
	cint_string SrcRolename;
	cint_string DstRolename;
	AssociationInfo(const char *src, const char *dst, const char *assoc);
	AssociationInfo(cint_string src, cint_string dst, cint_string assoc);
};

struct CompositionInfo
{
	cint_string parentRole;
	cint_string childRole;
	CompositionInfo(const char *pr, const char *cr);
	CompositionInfo(cint_string pr, cint_string cr);
};


static const int TARGETFROMPEER = 0;
static const int TARGETFROMCLASS = 1;
static const int CLASSFROMTARGET = 2;


class UdmPseudoObject
{
	
public:
	unsigned long ob_id;
	unsigned long dn_id;
	
	
	UdmPseudoObject();				//NULL object
	UdmPseudoObject(unsigned long dn_id, unsigned long ob_id);
	UdmPseudoObject& operator=(const UdmPseudoObject& frm);

	//error handling (mostly for errors resulting from operations 
	//being executed on invalid ids)
	static bool GetLastError(cint_string& buffer);
	
	//type
	bool type(cint_string &) const;
	bool stereotype(cint_string &value) const;
	bool uniqueId(unsigned long &) const;

	//IMPL API
	bool getParent(const cint_string &prole, UdmPseudoObject &value) const;
	bool setParent(UdmPseudoObject &parent, const cint_string &prole);
	bool detach();
	bool getChildren(const cint_string &crole, const cint_string &kind, UdmPseudoObjectS &value) const;
	bool setChildren(const cint_string &crole, const UdmPseudoObjectS &value);



	
	//associations
	bool getAssociation(const cint_string &ass_role, UdmPseudoObjectS &value, int mode = TARGETFROMPEER) const;
	bool setAssociation(const cint_string &ass_role, const UdmPseudoObjectS &value, int mode = TARGETFROMPEER);



	//attribute setters
	bool SetIntVal(const cint_string &name, long value);
	bool SetRealVal(const cint_string &name, double value);
	bool SetStrVal(const cint_string &name, const cint_string & value);
	bool SetBoolVal(const cint_string &name, bool value);

	//attribute getters
	bool GetIntVal(const cint_string &name, long &value);
	bool GetRealVal(const cint_string &name, double &value);
	bool GetStrVal(const cint_string &name, cint_string & value);
	bool GetBoolVal(const cint_string &name, long &value);
	
	//TOMI
	bool GetAdjacentObjects(UdmPseudoObjectS &value);
	bool GetAdjacentObjects(const cint_string &dst_type, UdmPseudoObjectS &value);
	bool GetAdjacentObjects(const cint_string &dst_type, const AssociationInfo & ass, UdmPseudoObjectS & value);

	bool GetParent(UdmPseudoObject &value);
	bool GetChildObjects(const cint_string & type, UdmPseudoObjectS &value);
	bool GetChildObjects(const CompositionInfo &ci, const cint_string & type, UdmPseudoObjectS &value);
	
	bool GetAssociationClassObjects(const UdmPseudoObject &dstObject, const AssociationInfo & ai, UdmPseudoObjectS &value); 
	bool GetPeersFromAssociationClassObject(UdmPseudoObjectS &value); 


	bool CreateObject(const cint_string &type, UdmPseudoObject &value );
	bool CreateLink(UdmPseudoObject &dst, const AssociationInfo& ass_type);

	bool DeleteObject();
};
