/*
Copyright (c) Vanderbilt University, 2000-2001
ALL RIGHTS RESERVED
Vanderbilt University disclaims all warranties with regard to this
software, including all implied warranties of merchantability
and fitness.  In no event shall Vanderbilt University be liable for
any special, indirect or consequential damages or any damages
whatsoever resulting from loss of use, data or profits, whether
in an action of contract, negligence or other tortious action,
arising out of or in connection with the use or performance of
this software.
*/

/*
	14/07/04	-	endre

			Changed the code generator to include both rolenames when generating the SetChildRole, SetParentRole
			calls.

	06/20/04	-	endre

			Fixed Code generator bug in case of multiple inheritence, when having an immediate and an indirect
			inheritence relationship between two classes.


	06/16/04	-	endre
			
			Changed the signature of GenerateCORBACPP to also include macro and cross-diagram arguments

	06/15/04	-	endre

			Fixed bug UDM-7(http://escher.isis.vanderbilt.edu/JIRA/browse/UDM-7)
			Removed DTD/XSD generator functions and using thise defined in DTDGen.h
	06/15/04	-	endre
		
			Fixed bug UDM-8( http://escher.isis.vanderbilt.edu/JIRA/browse/UDM-8)

	06/04/04	-	endre


			Workaround for the previous compiler bug. Typedef removed from the code generator.

	05/24/04	-	endre

			- unfortunattly compiler failes in generated code if the Udm::Object typedef is not present
	05/24/04	-	endre

			- redesigned the new Initilize(const Uml::Diagram&) function generator such that unique rolenames are no more
			required.
			- removed the typedef namespace::Object Udm::Object, thus there can be a class named 'Object' in custom
			namespaces without conflicting with Udm::Object
			- when using a macro definition, Udm.exe also generates an export file which make easier the DLL export/import
			of the generated classes


	05/06/04	-	endre

			- added Initialize(const Uml::Diagram& dgr) function which can initialize the static meta pointers in the API
			from an existing UML diagram
	04/22/04	-	endre

			added _sorted()  templated versions for functions returning instances of instantiated template classes:
				CrossAssocAttr, AssocAttr, AClassCrossAssocAttr, 
				AClassAssocAttr, ChildrenAttr, DerivedAttr, InstantiatedAttr
			The *_sorted(const Pred &) versions accept a templae class parameter which is supposed to order the resulting set<CLASS>


	04/03/04	-	endre

			changed the generator to generate virtual destructor for Visitor classes
	03/15/04	-	endre
			
			changed the internal dom attributes from using literal strings to constants
			defined in UdmDom namespace

	24/02/04	-	endre
			
			  changed the DTD/XSD generators to generate version information as processing instructions
	02/02/04	-	endre

			Changed the new CPP generator - splitted  the Initialize() function 
			in three smaller functions because it created problems with MSVC' limitations.
		
	15/01/04	-	endre
			Changed XSD Schema generator's root element selection condition: All classes, 
			which have no container classes but self are allowed to be root elements.

	12/12/03	-	endre

			Changed the cross class - cross diagram delimitator character from
			"_" to string "_cross_ph_". This allows '_' characters in classnames.

	01/17/02	-	endre
		
			Added constraints that prevent the generation
			of invalid code because of using reserved or duplicate
			attirubute/role names

	11/05/02	-	endre

			DTD generator changed; subelements 
	10/25/02	-	endre

			Added support for array defvalues and new attribute notation
	07/31/02	-	endre
		
		  C#, CORBA  code generators added

	06/20/02	-	endre
			
			changed .h code generator to support array attributes in case 
			of non-persistent attributes as well

	06/17/02	-	endre
			
			changed code generators to support attribute arrays:
				.h:	 when max is not equal to 1 or 0 for an attribute,
					the generated functions returns a AttrArr type attribute object,
					which contains a vector
				.dtd: in case of boolean vectors, the DTD attrib definition is not
					  <true;false> but CDATA. (Parsing is done by the DOM backend)

						
	04/22/02	-	endre
			
			  changed the attribute generation code to generate the __child_as 
			  composition child role attribute in the DTD as an enum of all possible childroles
			  instead of an open NMTOKENS. As requested by Jonathan.

  */

#include "Uml.h"
#include "UmlExt.h"
#include <time.h>
#include <map>


// assign a name to a role
// if name is assigned, use that name
// otherwise, if navigable, get the peer class name and append _assoc, _parent, _child or _children 
//									(for _parent, prepend ChildRole name, if applicable)
//			  if not navigable, name is arolexxx, prolexxx, or crolexxx, where xxx is the uniqueid

using namespace Uml;
	
string GetTime()	{
		time_t t;
		time(&t);
		char *c = ctime(&t);

		c[24] = '\0';
		return c;
}
void GenerateCreateCORBADiagram(const Uml::Diagram &diagram,  ostream &output);

string CheckAttrType(const Udm::StringAttr &p) {
				string pv = p;
				if(pv.compare("String") && pv.compare("Boolean") && 
					pv.compare("Integer") && pv.compare("Real")) {
					throw udm_exception("invalid attribute type '"+pv+"'");
				}
				return pv;
}

// C# generators

void GenerateCSApi(const Uml::Diagram &diagram,  ostream &output, string fname)
{
	string hname = diagram.name();
	string::iterator i;
	for(i = hname.begin(); i != hname.end(); i++) 
	{
		if(!isalnum(*i)) *i = '_';
	}
	output << "// CS API file " << fname << ".cs generated from diagram " << (string)diagram.name() << endl;
	output << "// generated on " << GetTime().c_str() << endl << endl;

	output << "using System;"	<< endl;
	output << "using System.Collections;"	<< endl <<endl << endl;
	output << "namespace " << hname << endl << "{" << endl << endl;

	set<Uml::Class> classes = diagram.classes();
	set<Uml::Class>::iterator c;

	for ( c = classes.begin(); c!= classes.end(); c++)
	{
		Uml::Class cl = *c;
		set<Uml::Class> bases = cl.baseTypes();
		
		if (bases.size() > 1) 
			throw udm_exception("C# UDM does not support multiple inheritence!");
		
		bool derived = (bases.size() > 0);
		
		
		//class declaration, inheritence
		if (derived)
		{
			Uml::Class base = *(bases.begin());
			output <<"\tpublic class " << (string)cl.name() << " : " << (string)base.name() << endl;
		}
		else
			output <<"\tpublic class " << (string) cl.name() << " : " << "Udm.Object" << endl;

		string cn = (string)cl.name();

		//static meta field
		output << "\t{" << endl;;
		if (derived)
			output <<"\t\tpublic static new Uml.Class meta;"<<endl;
		else
			output <<"\t\tpublic static Uml.Class meta;"<<endl;

		//static NULL field
		output <<"\t\tpublic static readonly new " << cn << " NULL = new " << cn << "();" <<endl;
		output << endl;

		//constructors
		output <<"\t\tpublic " << cn << "() {}" << endl;
		output <<"\t\tpublic " << cn << "(Udm.ObjectImpl impl) : base(impl) {}" <<endl;
		output <<"\t\tpublic " << cn << "(" << cn <<" master) : base(master) {}" <<endl;
		output << endl;
		
		//creators
		if (derived)
			output <<"\t\tpublic static new " << cn  << " Create(Udm.Object parent)" << endl;
		else
			output <<"\t\tpublic static " << cn  << " Create(Udm.Object parent)" << endl;

		output <<"\t\t{" <<endl;
		output <<"\t\t\treturn " << cn << ".Cast((Udm.Object.Create(ref meta, parent, Uml.CompositionChildRole.NULL)));" <<endl;
		output <<"\t\t}" <<endl;
		
		if (derived)
			output <<"\t\tpublic static new " << cn  << " Create(Udm.Object parent, Uml.CompositionChildRole ccr)" << endl;
		else
			output <<"\t\tpublic static " << cn  << " Create(Udm.Object parent, Uml.CompositionChildRole ccr)" << endl;

		output <<"\t\t{" <<endl;
		output <<"\t\t\treturn " << cn << ".Cast((Udm.Object.Create(ref meta, parent, ccr)));" <<endl;
		output <<"\t\t}" <<endl;
		
		//cast
		if (derived)
			output <<"\t\tpublic static new " << cn  << " Cast(Udm.Object o)" << endl;
		else
			output <<"\t\tpublic static " << cn  << " Cast(Udm.Object o)" << endl;

		output <<"\t\t{" <<endl;
		output <<"\t\t\tif (Uml.UmlExt.IsObjectDerivedFromClass(o, meta))" << endl;
		output <<"\t\t\t\treturn new " << cn << "(o.__impl());" << endl;
		output <<"\t\t\telse throw new Udm.udm_exception(\"Invalid cast!\");" << endl;
		output <<"\t\t}" <<endl;

		
		

		set<Uml::Attribute> attributes = cl.attributes();
		set<Uml::Attribute>::iterator a;
		for ( a = attributes.begin(); a != attributes.end(); a++)
		{
			string aname = (*a).name();
			string atype = (*a).type();
			string cs_atype;
			bool multiple = ((long)(*a).max() != 0) && ((long)(*a).max() != 1);

			if (atype == "Boolean") cs_atype = "bool";
			else if (atype == "Real") cs_atype = "double";
			else if (atype == "Integer") cs_atype = "long";
			else if (atype == "String") cs_atype = "string";
			else throw new udm_exception(string("Invalid attribute type: ") + atype);
			//check for the same attribute in ascendants
				
			output << "\t\tpublic static Uml.Attribute meta_" << aname << ";"<< endl;
			if (!multiple)
				output << "\t\tpublic " << cs_atype << " " << aname << endl;
			else
				output << "\t\tpublic " << cs_atype << "[] " << aname << endl;

			output << "\t\t{" << endl;
			if (!multiple)
			{
				output << "\t\t\tget {return impl.get" << atype << "Attribute(meta_" << aname <<");}" << endl;
				output << "\t\t\tset {impl.set" << atype << "Attribute(meta_"<< aname <<", value);}" <<endl;
			}
			else
			{
				output << "\t\t\tget {return impl.get" << atype << "Attributes(meta_" << aname <<");}" << endl;
				output << "\t\t\tset {impl.set" << atype << "Attributes(meta_"<< aname <<", value);}" <<endl;
			}
			output << "\t\t}" << endl;
			
		}

		output << endl;
		//assoc. roles
		set<Uml::AssociationRole> aroles = cl.associationRoles();
		set<Uml::AssociationRole>::iterator b_i;
		for (b_i = aroles.begin(); b_i != aroles.end(); b_i++)
		{
			Uml::AssociationRole arole = Uml::theOther((*b_i));
			string arname = MakeRoleName(arole);
			bool multiple = ((arole.max() != 1) && (arole.max() != 0));
			Uml::Class target = arole.target();
			string tname = target.name();
			

			
			Uml::Association parent = arole.parent();
			Uml::Class assocClass = parent.assocClass();

			
			if (assocClass)
				output << "\t\tpublic static Uml.AssociationRole meta_" << arname << ", meta_" <<arname <<"_rev ;" << endl;
			else
				output << "\t\tpublic static Uml.AssociationRole meta_" << arname << ";" << endl;
			
			if (!multiple)
			{
			
				
				output << "\t\tpublic " << tname << " " << arname << endl;
				output << "\t\t{" << endl;

				if (assocClass)
				{
					output << "\t\t\tget{ return (" << tname <<")(getPointer(meta_" << arname << ", \"" << diagram.name() << "\", Udm.UdmAssocMode.CLASSFROMTARGET));}" <<endl; 
					output << "\t\t\tset{ setPointer(meta_" << arname << ", value, Udm.UdmAssocMode.CLASSFROMTARGET);}" <<endl; 
				}
				else
				{
					output << "\t\t\tget{ return (" << tname <<")(getPointer(meta_" << arname << ", \"" << diagram.name() << "\", Udm.UdmAssocMode.TARGETFROMPEER));}" <<endl; 
					output << "\t\t\tset{ setPointer(meta_" << arname << ", value, Udm.UdmAssocMode.TARGETFROMPEER);}" <<endl; 
				}

				output << "\t\t}" << endl;

			}
			else
			{
				output << "\t\tpublic Hashtable " <<arname << endl;
				output << "\t\t{" << endl;
				if (assocClass)
				{
					output << "\t\t\tget{ return (getPointers(meta_" << arname << ", \"" << diagram.name() << "\", Udm.UdmAssocMode.CLASSFROMTARGET));}" <<endl; 
					output << "\t\t\tset{ setPointers(meta_" << arname << ", value, Udm.UdmAssocMode.CLASSFROMTARGET);}" <<endl; 

				}
				else
				{
					output << "\t\t\tget{ return (getPointers(meta_" << arname << ", \"" << diagram.name() << "\", Udm.UdmAssocMode.TARGETFROMPEER));}" <<endl; 
					output << "\t\t\tset{ setPointers(meta_" << arname << ", value, Udm.UdmAssocMode.TARGETFROMPEER);}" <<endl; 
				}
				output << "\t\t}" << endl;

			}
		}

		//assoc. class
		Uml::Association ass = cl.association();
		if (ass)
		{
			set<Uml::Class>::iterator bas_i;
			for (bas_i = bases.begin(); bas_i != bases.end(); bas_i++)
			{
				Uml::Association new_ass = bas_i->association();
				if (new_ass)
					throw udm_exception("An association class has two associations?");
			}
			set<Uml::AssociationRole> ass_roles = ass.roles();
			set<Uml::AssociationRole>::iterator ar_i;
			for (ar_i = ass_roles.begin(); ar_i != ass_roles.end(); ar_i++)
			{
				Uml::AssociationRole arole = *ar_i;
				Uml::Class target = arole.target();
				string tname   = target.name();
				string arname  = MakeRoleName(arole);
				
				output << "\t\tpublic static Uml.AssociationRole meta_" << arname << "_end_;" << endl;
				output << "\t\tpublic " << tname << " " << arname  << endl;
				output << "\t\t{" << endl;
				output << "\t\t\tget{ return (" << tname <<")(getPointer(meta_" << arname << "_end_, \"" << diagram.name() << "\", Udm.UdmAssocMode.TARGETFROMCLASS));}" <<endl; 
				output << "\t\t\tset{ setPointer(meta_" << arname << "_end_, value, Udm.UdmAssocMode.TARGETFROMCLASS);}" <<endl; 
				output << "\t\t}" << endl;

				output << endl;


			}

		}
		output << endl;
		set<Uml::Class> childrenkinds;
		//parentroles
		set<Uml::CompositionParentRole> cprs = cl.parentRoles();
		set<Uml::CompositionParentRole>::iterator c_i;
		for (c_i = cprs.begin(); c_i != cprs.end(); c_i++)
		{
			Uml::CompositionChildRole ccr = Uml::theOther(*c_i);
			string ccr_name = MakeRoleName(ccr);
				//ccr.name();
			string tname = ((Uml::Class)ccr.target()).name();
			
			childrenkinds.insert((Uml::Class)ccr.target());
			bool multiple = ((ccr.max() != 0 ) && (ccr.max() != 1 ));
			
			output << "\t\tpublic static Uml.CompositionChildRole meta_" << ccr_name << ";" <<endl;

			if (!multiple)
			{
				output << "\t\tpublic " << tname << " " << ccr_name << endl;
				output << "\t\t{" << endl;
				output << "\t\t\tget{ return (" << tname <<")(getChild(meta_" << ccr_name << ", Uml.Class.NULL, \"" << diagram.name() << "\"));}" <<endl; 
				output << "\t\t\tset{ setChild(meta_" << ccr_name << ", value);}" <<endl; 
				output << "\t\t}" << endl;
			}
			else
			{
				output << "\t\tpublic Hashtable " <<ccr_name << endl;
				output << "\t\t{" << endl;
				output << "\t\t\tget{ return (getChildren(meta_" << ccr_name << ", Uml.Class.NULL, \"" << diagram.name() << "\"));}" <<endl; 
				output << "\t\t\tset{ setChildren(meta_" << ccr_name << ", value);}" <<endl; 
				output << "\t\t}" << endl;

			}
		}
		//__kind_children calls
		set<Uml::Class>::iterator f_i, g_i;
		for (f_i = classes.begin(); f_i != classes.end(); f_i++)
		{
			for (g_i = childrenkinds.begin(); g_i != childrenkinds.end(); g_i++)
			{
				if (Uml::IsDerivedFrom(*g_i, *f_i) || Uml::IsDerivedFrom(*f_i, *g_i)) 
				{
					string kindname = f_i->name();
					output << endl;
					output << "\t\tpublic Hashtable " << kindname << "_kind_children" << endl;
					output << "\t\t{" << endl;
					output << "\t\t\tget{ return (getChildren(Uml.CompositionChildRole.NULL, " << kindname << ".meta, \"" << diagram.name() << "\"));}" <<endl; 
					//no setter for kind_children calls
					output << "\t\t}" << endl;
					break;
				}
			}
		}

		output << endl;
		
		//childroles
		set<Uml::CompositionChildRole> ccrs = cl.childRoles();
		set<Uml::CompositionChildRole>::iterator d_i;
		bool was_parent = false;

		for (d_i = ccrs.begin(); d_i != ccrs.end(); d_i++)
		{
			Uml::CompositionParentRole cpr = Uml::theOther(*d_i);
			string cpr_name = MakeRoleName(cpr);
			string tname = ((Uml::Class)cpr.target()).name();


			bool new_required = false;
			new_required = derived && (cpr_name == "parent");

			
			if (!was_parent && cpr_name == "parent") was_parent = true;
			output << "\t\tpublic static Uml.CompositionParentRole meta_" << cpr_name << ";" <<endl;

			if (new_required)
				output << "\t\tpublic new " << tname << " " << cpr_name << endl;
			else
				output << "\t\tpublic " << tname << " " << cpr_name << endl;
			output << "\t\t{" << endl;
			output << "\t\t\tget{ return (" << tname <<")(getParent(meta_" << cpr_name << ", \"" << diagram.name() << "\"));}" <<endl; 
			output << "\t\t\tset{ setParent(meta_" << cpr_name << ", value);}" <<endl; 
			output << "\t\t}" << endl;
		}

		if (!was_parent)
		{
			if (derived) 
				output << "\t\tpublic new Udm.Object parent" <<endl;
			else
				output << "\t\tpublic Udm.Object parent" <<endl;

			output << "\t\t{" << endl;
			output << "\t\t\tget{ return getParent(Uml.CompositionParentRole.NULL, \"" << diagram.name() <<"\");}" <<endl;
			output << "\t\t\tset{ setParent(Uml.CompositionParentRole.NULL, value);}" << endl;
			output << "\t\t}" << endl;

		}


		output << "\t}" << endl;//end of class



	}
	
	
}

void GenerateCSInit(const Uml::Diagram &diagram,  ostream &output, string fname)
{
	output << "\t"		<< "public class Init" << endl;
	output << "\t"		<< "{" << endl; 
	output << "\t\t"		<< "public static bool initialized = false;" << endl;
	output << "\t\t"		<< "public static Udm.DataNetwork dn = null;" << endl;
	output << "\t\t"		<< "public static Uml.Diagram umldiagram = null;" << endl;
	output << "\t\t"		<< "static Init()" << endl;
	output << "\t\t"		<< "{"<<endl;
	output << "\t\t\t"			<< "if(initialized) return;" << endl;
	output << "\t\t\t"			<< "initialized = true;" << endl;
	output << "\t\t\t"			<< "if(!Uml.Init.initialized) throw new Udm.udm_exception(\"UML Diagram is not yet initialized!\");" << endl;
	output << "\t\t\t"			<< "umldiagram = Uml.UmlExt.CreateDiagram();" << endl;

	set<Uml::Class> classes = diagram.classes();
	set<Uml::Class>::iterator c;

	//create classes
	for(c = classes.begin(); c != classes.end(); c++ )
	{
		Uml::Class cl = *c;
		output << "\t\t\t" << cl.name() << ".meta = Uml.UmlExt.CreateClass();" << endl;
	}

	//create attributes
	for(c = classes.begin(); c != classes.end(); c++ )	
	{
		Uml::Class cl = *c;
		set<Uml::Attribute> attributes = cl.attributes();

		//todo Non-persistent attribute support for C# UDM
	
		for( set<Uml::Attribute>::iterator i = attributes.begin(); i != attributes.end(); i++) 
		{
			output << "\t\t\t" << cl.name() << ".meta_" << (*i).name() << " = Uml.UmlExt.CreateAttribute();" << endl;
		}
	}

	//create association roles
	for(c = classes.begin(); c != classes.end(); c++ ) 
	{
		Uml::Class cl = *c;
		set<Uml::AssociationRole> assocs = cl.associationRoles();
		for( set<Uml::AssociationRole>::iterator i = assocs.begin(); i != assocs.end(); i++) 
		{
			string aname(MakeRoleName(Uml::theOther(*i)));
			output << "\t\t\t" << cl.name() << ".meta_" << aname << " = Uml.UmlExt.CreateAssociationRole();" << endl;
		}
	}

	//create composition parent roles
	for(c = classes.begin(); c != classes.end(); c++ ) 
	{
		Uml::Class cl = *c;
		set<Uml::CompositionChildRole> children = cl.childRoles();
		for( set<Uml::CompositionChildRole>::iterator i = children.begin(); i != children.end(); i++) 
		{
			string aname(MakeRoleName(Uml::theOther(*i)));
			output << "\t\t\t" << cl.name() << ".meta_" << aname << " = Uml.UmlExt.CreateCompositionParentRole();" << endl;
		}
	}

	//create composition child roles

	for(c = classes.begin(); c != classes.end(); c++ ) 
	{
		Uml::Class cl = *c;
		set<Uml::CompositionParentRole> children = cl.parentRoles();
		for( set<Uml::CompositionParentRole>::iterator i = children.begin(); i != children.end(); i++) 
		{
			string aname(MakeRoleName(Uml::theOther(*i)));
			output << "\t\t\t" << cl.name() << ".meta_" << aname << " = Uml.UmlExt.CreateCompositionChildRole();" << endl;
		}
	}

	//end of creations,

	output << "\t\t\t" << "Udm.Object.SolveMetaDependencies();"<< endl;
	
	
	//init part


	//diagram
	output << "\t\t\tUml.UmlExt.InitDiagram(umldiagram, \"" << diagram.name() << "\");" << endl << endl;

	//classes
	for(c = classes.begin(); c != classes.end(); c++ )
	{
		Uml::Class cl = *c;
		string ster;
		if(((string)cl.stereotype()).length()) ster = string(", \"")+(string)cl.stereotype()+"\"";

		string from;
		if(((string)cl.from()).length()) from = string(", \"")+(string)cl.from()+"\"";

		output << "\t\t\tUml.UmlExt.InitClass(" << cl.name() << ".meta, umldiagram, \"" <<
			cl.name() << "\", " << (cl.isAbstract() ? "true" : "false")<< ster << from << ");" << endl;

	}
	
	output << "\t\t" << endl;

	//atributes
	for(c = classes.begin(); c != classes.end(); c++ )	
	{
		Uml::Class cl = *c;
	
//		Uml::Class tempcl(Uml::classByName(tempdiagram, cl.name()));
		set<Uml::Attribute> attributes = cl.attributes();
		
		for( set<Uml::Attribute>::iterator i = attributes.begin(); i != attributes.end(); i++) 
		{
			output << "\t\t\tUml.UmlExt.InitAttribute(" << cl.name() << ".meta_" << (*i).name() << ", " <<
				cl.name() << ".meta, \"" << (*i).name() << "\", \"" << (*i).type() << "\", " <<
				((*i).nonpersistent() ? "true" : "false") << ", " << (*i).min() << ", " << (*i).max() << ");" << endl;
		}
	}
		
	output << "\t\t" << endl;

	//associations
	{
		const set<Uml::Association> &asss = diagram.associations();
		set<Uml::Association>::const_iterator a;
		for(a = asss.begin(); a != asss.end(); a++ )	
		{
			set<Uml::AssociationRole> ar = a->roles();
			ASSERT(ar.size() == 2);
//	{
			output << "\t\t\t{" << endl;
//		Uml::Association ass = Uml::CreateAssociation();
			output << "\t\t\t\tUml.Association ass = Uml.UmlExt.CreateAssociation();" << endl;

//		Uml::InitAssociation(ass, diagram, "AtoB");
// or : Uml::InitAssociationClass(ass, C::meta);
			Uml::Class aclass = a->assocClass();
			output << "\t\t\t\tUml.UmlExt.InitAssociation(ass , umldiagram, \"" << a->name() << "\");" << endl;
			if(aclass) {
				output << "\t\t\tUml.UmlExt.InitAssociationClass(ass, " << aclass.name() << ".meta);" << endl;
			}

//		Uml::InitAssociationRole(A::meta_dst, ass, "dst", 0, 1, B::meta);
//								C:: meta_dst_end_ = B:meta_src_rev = A::meta_dst;
			set<Uml::AssociationRole> roles = a->roles();
			for( set<Uml::AssociationRole>::iterator i = roles.begin(); i != roles.end(); i++) {
					Uml::AssociationRole zz	= Uml::theOther(*i);
					string aname(MakeRoleName(zz));
					Uml::Class cl = i->target();
					output << "\t\t\t\tUml.UmlExt.InitAssociationRole(" << cl.name() << ".meta_" << aname 
						<< ", ass, \"" << aname << "\", " <<
						(zz.isNavigable() ? "true, " : "false, ") <<
						(zz.isPrimary() ? "true, " : "false, ") <<
						zz.min() << ", " << zz.max() << ", " << ((Uml::Class)zz.target()).name() << ".meta);" << endl;
					Uml::Class aclass = a->assocClass();
					if(aclass) {
						Uml::Class cl2 = zz.target();
						output << "\t\t\t" << aclass.name() << ".meta_" << aname << "_end_ = " <<
											cl2.name() << ".meta_" << MakeRoleName(*i) << "_rev = " << 
											cl.name() << ".meta_" << aname << ";" << endl;
					}
			}
//	}
			output << "\t\t\t}" << endl;
		}
	}

	//compositions

	{
		const set<Uml::Composition> &comps = diagram.compositions();
		set<Uml::Composition>::const_iterator c;
		for(c = comps.begin(); c != comps.end(); c++ )	{
//	{
			output << "\t\t\t{" << endl;
//		Uml::Composition comp = Uml::CreateComposition();
			output << "\t\t\t\tUml.Composition comp = Uml.UmlExt.CreateComposition();" << endl;

//		Uml::InitComposition(comp, diagram, "DtoA");
			output << "\t\t\t\tUml.UmlExt.InitComposition(comp , umldiagram, \"" << 
				c->name() << "\");" <<  endl;
			output << "\t\t" << endl;

			Uml::CompositionParentRole x1 = (*c).parentRole();
			Uml::CompositionChildRole x2 = (*c).childRole();

//		Uml::InitCompositionParentRole(A::meta_parent, comp, "parent", D::meta);
			Uml::CompositionParentRole zzp = c->parentRole();
			Uml::CompositionChildRole zzc = c->childRole();
			string aname(MakeRoleName(zzp));
			output << "\t\t\t\tUml.UmlExt.InitCompositionParentRole(" << ((Uml::Class)zzc.target()).name() << ".meta_" << aname << 
				", comp, \""<< aname << "\", " <<
				(zzp.isNavigable() ? "true, " : "false, ") <<
				((Uml::Class)zzp.target()).name() << ".meta);" << endl;
					
//		Uml::InitCompositionChildRole(D::meta_as, comp, "as", true, 0, -1, A::meta); 
			string cname(MakeRoleName(zzc));
			output << "\t\t\t\tUml.UmlExt.InitCompositionChildRole(" << ((Uml::Class)zzp.target()).name() << ".meta_" << cname << 
				", comp, \""<< MakeShortRoleName(zzc) << "\", " <<
					(zzc.isNavigable() ? "true, " : "false, ") <<
				zzc.min() << ", " << zzc.max() << ", " <<
				((Uml::Class)zzc.target()).name() << ".meta);" << endl;
//	}
			output << "\t\t\t}" << endl;
		}
	}


	//inheritence

//		Uml::AddInheritance(A::meta, C::meta);
	for(c = classes.begin(); c != classes.end(); c++ ) 
	{
		Uml::Class cl = *c;
		set<Uml::Class> st = cl.subTypes();
		for( set<Uml::Class>::iterator i = st.begin(); i != st.end(); i++) 
		{
			output << "\t\t\tUml.UmlExt.AddInheritance(" << cl.name() << ".meta, " << (*i).name() << ".meta);" << endl;
		}
	}			

	output << endl;

	output << "\t\t\t" << "Static.StaticDataNetwork sdn = new Static.StaticDataNetwork(umldiagram);" << endl;
	output << "\t\t\t" << "sdn.CreateFromRootObj(\"" << diagram.name() <<"\", umldiagram, Udm.BackEndSemantics.CHANGES_LOST_DEFAULT);" << endl;
	output << "\t\t\t" << "dn = sdn;" << endl;

	output << "\t\t"		<< "}"<<endl;		//end of Init.init();
	output << "\t"	   << "}"<<endl;			//end of class Init();
	output << "}" << endl;						//end of namespace
}
void GenerateNewCSInit(const Uml::Diagram &diagram,  ostream &output, string fname)
{
	output << "\t"		<< "public class Init" << endl;
	output << "\t"		<< "{" << endl; 
	output << "\t\t"		<< "public static bool initialized = false;" << endl;
	output << "\t\t"		<< "public static Udm.DataNetwork dn = null;" << endl;
	output << "\t\t"		<< "public static Uml.Diagram umldiagram = null;" << endl;
	output << "\t\t"		<< "static Init()" << endl;
	output << "\t\t"		<< "{"<<endl;
	output << "\t\t\t"			<< "if(initialized) return;" << endl;
	output << "\t\t\t"			<< "initialized = true;" << endl;
	output << "\t\t\t"		<< "if(!Uml.Init.initialized) throw new Udm.udm_exception(\"UML Diagram is not yet initialized!\");" << endl;


	string hname = diagram.name();

	output << "\t\t\t"			<< "dn = new Static.StaticDataNetwork(Uml.Init.dn.GetRootMeta());" << endl;
	output << "\t\t\t"			<< "dn.CreateNew(\"" << hname << ".mem\", \"\",ref Uml.Diagram.meta, Udm.BackEndSemantics.CHANGES_LOST_DEFAULT);"<< endl;
	output << "\t\t\t"			<< "umldiagram = Uml.Diagram.Cast(dn.GetRootObject());" << endl;
	output << "\t\t\t"			<< "umldiagram.name =\"" << hname << "\";" << endl;

	
	set<Uml::Class> classes = diagram.classes();
	set<Uml::Class>::iterator c;

	//classes
	for(c = classes.begin(); c != classes.end(); c++ )
	{
		Uml::Class cl = *c;
		output << "\t\t\t" << cl.name() << ".meta = Uml.Class.Create(umldiagram);" << endl;
	}

	//attributes
	for(c = classes.begin(); c != classes.end(); c++ )	
	{
		Uml::Class cl = *c;
		set<Uml::Attribute> attributes = cl.attributes();
		//todo: non-persistent attributes
	
		for( set<Uml::Attribute>::iterator i = attributes.begin(); i != attributes.end(); i++) 
		{
			output << "\t\t\t" << cl.name() << ".meta_" << (*i).name() << " = Uml.Attribute.Create(" << cl.name()<<".meta);" << endl;
		}
	}


	//class attributes
	for(c = classes.begin(); c != classes.end(); c++ )
	{
		Uml::Class cl = *c;
		string ster;
		if(((string)cl.stereotype()).length()) ster = string(", \"")+(string)cl.stereotype()+"\"";
	
		output << "\t\t\t" << cl.name() << ".meta.name = \"" << cl.name() << "\";" <<endl;
		output << "\t\t\t" << cl.name() << ".meta.isAbstract = " << (cl.isAbstract() ? "true" : "false") << ";" << endl;
		
		if (!ster.empty()) 
				output << "\t\t\t" << cl.name() << ".meta.stereotype = \"" << cl.stereotype() << "\";" <<endl;

	}
	
	//attribute attributes ;-)
	for(c = classes.begin(); c != classes.end(); c++ )	
	{
		Uml::Class cl = *c;
//		Uml::Class tempcl(Uml::classByName(tempdiagram, cl.name()));

		set<Uml::Attribute> attributes = cl.attributes();

		for(set<Uml::Attribute>::iterator att_i = attributes.begin(); att_i != attributes.end(); att_i++)
			//nonpersistent is false for attributes coming from the real diagram
			att_i->nonpersistent() = false;
			

		for( set<Uml::Attribute>::iterator i = attributes.begin(); i != attributes.end(); i++) 
		{
		
			output << "\t\t\t" << cl.name() << ".meta_" << (*i).name() << ".name = \"" << (*i).name() << "\";" << endl;
			output << "\t\t\t" << cl.name() << ".meta_" << (*i).name() << ".type = \"" << (*i).type() << "\";" << endl;
			output << "\t\t\t" << cl.name() << ".meta_" << (*i).name() << ".min = " << (*i).min() << ";" << endl;
			output << "\t\t\t" << cl.name() << ".meta_" << (*i).name() << ".max = " << (*i).max() << ";" << endl;
			output << "\t\t\t" << cl.name() << ".meta_" << (*i).name() << ".nonpersistent = " << ((*i).nonpersistent() ? "true" : "false") << ";" << endl;
		}
	}

	//associations

	{
		const set<Uml::Association> &asss = diagram.associations();
		set<Uml::Association>::const_iterator a;
		for(a = asss.begin(); a != asss.end(); a++ )
		{
			set<Uml::AssociationRole> ar = a->roles();
			ASSERT(ar.size() == 2);

			output << "\t\t\t{" << endl;
			output << "\t\t\t\tUml.Association ass = Uml.Association.Create(umldiagram);" << endl;

			Uml::Class aclass = a->assocClass();
			if (!((string)a->name()).empty())
				output << "\t\t\t\tass.name = \"" << a->name() << "\";" <<endl;
			if(aclass) 
			{
				output << "\t\t\t\tass.assocClass = " << aclass.name() << ".meta;" << endl;
			}

			set<Uml::AssociationRole> roles = a->roles();
			for( set<Uml::AssociationRole>::iterator i = roles.begin(); i != roles.end(); i++) {
					Uml::AssociationRole zz	= Uml::theOther(*i);
					string aname(MakeRoleName(zz));
					Uml::Class cl = i->target();

					output << "\t\t\t\t" << cl.name() << ".meta_" << aname << " = Uml.AssociationRole.Create(ass);" <<endl;
					output << "\t\t\t\t" << cl.name() << ".meta_" << aname << ".name = \"" << aname << "\";" <<endl;
					output << "\t\t\t\t" << cl.name() << ".meta_" << aname << ".min = " << zz.min() << ";" <<endl;
					output << "\t\t\t\t" << cl.name() << ".meta_" << aname << ".max = " << zz.max() << ";" <<endl;
					output << "\t\t\t\t" << cl.name() << ".meta_" << aname << ".isNavigable = " << (zz.isNavigable() ? "true":"false") << ";" <<endl;
					output << "\t\t\t\t" << cl.name() << ".meta_" << aname << ".isPrimary = " << (zz.isPrimary() ? "true":"false") << ";" <<endl;
					output << "\t\t\t\t" << cl.name() << ".meta_" << aname << ".target = " << ((Uml::Class)zz.target()).name() << ".meta;" <<endl;


					
					Uml::Class aclass = a->assocClass();
					if(aclass) 
					{
						Uml::Class cl2 = zz.target();
						output << "\t\t\t\t" << aclass.name() << ".meta_" << aname << "_end_ = " <<
											cl2.name() << ".meta_" << MakeRoleName(*i) << "_rev = " << 
											cl.name() << ".meta_" << aname << ";" << endl;
					}
			}
			output << "\t\t\t}" << endl;
		}
	}


	//compositions

	{
		const set<Uml::Composition> &comps = diagram.compositions();
		set<Uml::Composition>::const_iterator c;
		for(c = comps.begin(); c != comps.end(); c++ )	
		{
			output << "\t\t\t{" << endl;
			output << "\t\t\t\tUml.Composition comp = Uml.Composition.Create(umldiagram);" << endl;

			if (!((string)c->name()).empty())
				output << "\t\t\t\tcomp.name =\"" << c->name() << "\";" <<endl;
			output << "\t\t\t\tcomp.nonpersistent =" << (c->nonpersistent() ? "true" : "false") << ";" << endl;


			output << "\t\t" << endl;

			Uml::CompositionParentRole x1 = (*c).parentRole();
			Uml::CompositionChildRole x2 = (*c).childRole();

			Uml::CompositionParentRole zzp = c->parentRole();
			Uml::CompositionChildRole zzc = c->childRole();
			string aname(MakeRoleName(zzp));

			output << "\t\t\t\t" << ((Uml::Class)zzc.target()).name() << ".meta_" << aname <<
				" = Uml.CompositionParentRole.Create(comp);" << endl;
			output << "\t\t\t\t" << ((Uml::Class)zzc.target()).name() << ".meta_" << aname <<
				".name = \"" << aname << "\";" << endl;
			output << "\t\t\t\t" << ((Uml::Class)zzc.target()).name() << ".meta_" << aname <<
				".isNavigable = " << (zzp.isNavigable() ? "true;" : "false;") << endl;
			output << "\t\t\t\t" << ((Uml::Class)zzc.target()).name() << ".meta_" << aname <<
				".target = " << ((Uml::Class)zzp.target()).name() << ".meta;" << endl;




			string cname(MakeRoleName(zzc));
			output << "\t\t\t\t" << ((Uml::Class)zzp.target()).name() << ".meta_" << cname << 
				" = Uml.CompositionChildRole.Create(comp);" << endl;

			output << "\t\t\t\t" << ((Uml::Class)zzp.target()).name() << ".meta_" << cname << 
				".name = \"" << MakeShortRoleName(zzc) << "\";" << endl;
			output << "\t\t\t\t" << ((Uml::Class)zzp.target()).name() << ".meta_" << cname << 
				".min = " << zzc.min() << ";"<< endl;
			output << "\t\t\t\t" << ((Uml::Class)zzp.target()).name() << ".meta_" << cname << 
				".max = " << zzc.max() << ";"<< endl;
			output << "\t\t\t\t" << ((Uml::Class)zzp.target()).name() << ".meta_" << cname << 
				".isNavigable = " << (zzc.isNavigable() ? "true;" : "false;") << endl;
			output << "\t\t\t\t" << ((Uml::Class)zzp.target()).name() << ".meta_" << cname << 
				".target = " << ((Uml::Class)zzc.target()).name() << ".meta;" << endl;

			output << "\t\t\t}" << endl;
		}
	}

	for(c = classes.begin(); c != classes.end(); c++ ) 
	{
		Uml::Class cl = *c;
		set<Uml::Class> st = cl.subTypes();
		if (st.size())
		{
			output << "\t\t\t" <<"Hashtable " << (string)cl.name() << "_meta_subtypes = new Hashtable();" << endl;
			for( set<Uml::Class>::iterator i = st.begin(); i != st.end(); i++) 
			{
			
				output << "\t\t\t" << cl.name() << "_meta_subtypes[" << (*i).name() << ".meta] = " <<(*i).name() << ".meta;" << endl;
			}
			output << "\t\t\t" << cl.name() << ".meta.subTypes = " << cl.name() << "_meta_subtypes;" << endl;
			output << endl;
		}
	}			




	output << "\t\t"		<< "}"<<endl;		//end of Init.init();
	output << "\t"	   << "}"<<endl;			//end of class Init();
	output << "}" << endl;	//end of namespace
}

void GenerateCS(const Uml::Diagram &diagram,  ostream &output, string fname)
{
	GenerateCSApi(diagram,  output, fname);
	GenerateCSInit(diagram, output, fname);
}
void GenerateNewCS(const Uml::Diagram &diagram,  ostream &output, string fname)
{
	GenerateCSApi(diagram,   output, fname);
	GenerateNewCSInit(diagram, output, fname);
}
//////

typedef vector<Uml::Class> UmlClasses;
void GenerateHExport(const Uml::Diagram &diagram, ostream &output, string fname,  const string& macro)
{
	string hname = diagram.name();
	string::iterator i;
	for(i = hname.begin(); i != hname.end(); i++) 
	{
		if(!isalnum(*i)) *i = '_';
	}
	
	string uhname = "MOBIES_" + fname + "_H";
	string hasdll_string = hname + "_HAS_DLL";
	string export_string = hname + "_EXPORT";
	
	output << "#ifndef " << uhname <<endl;
	output << "#define " << uhname <<endl;

	output << "#if defined (" << hasdll_string << ") && (" << hasdll_string << " == 1)" <<endl;
	output << "#	if defined (" << export_string << " )" << endl;
	output << "#		define " << macro << " __declspec(dllexport)" << endl;
	output << "#	else" << endl;
	output << "#		define " << macro << " __declspec(dllimport)" << endl;
	output << "#	endif /* " << export_string << " */" << endl;
	output << "#else" <<endl;
	output << "#	define " << macro << endl;
	output << "#endif /*" << export_string << "*/" << endl;


	output << "#endif /* " << uhname << " */ " << endl;



};

void GenerateH(const Uml::Diagram &diagram, ostream &output, string fname, bool visitor_sup, const Uml::Diagram& cross_dgr, const string& macro)	
{
		string hname = diagram.name();
		
		string::iterator i;
		for(i = hname.begin(); i != hname.end(); i++) {
			if(!isalnum(*i)) *i = '_';
		}
		string uhname = "MOBIES_" + fname + "_H";

		for(i = uhname.begin(); i != uhname.end(); i++) {
			*i = toupper(*i);
		}

		output << "#ifndef "<< uhname << endl <<"#define " << uhname << endl;
		output << "// header file " << fname << ".h generated from diagram " << (string)diagram.name() << endl;
		output << "// generated on " << GetTime().c_str() << endl << endl;


		output << "#ifndef MOBIES_UDMBASE_H" << endl;
		output << "#include \"UdmBase.h\""	<< endl;
		output << "#endif"	<< endl << endl;

		

		if (cross_dgr)
		{
			set<Uml::Class> cross_classes = cross_dgr.classes();
			

			map<string, UmlClasses> cr_cls_nsp;

			for(set<Uml::Class>::iterator i = cross_classes.begin(); i != cross_classes.end(); i++)
			{
				string from = i->from();

				if ((from == hname)|| (from.size()==0)) continue;
				vector<Uml::Class> & clss = cr_cls_nsp[from];
				clss.push_back(*i);
			};

			for(map<string, UmlClasses>::iterator j = cr_cls_nsp.begin(); j != cr_cls_nsp.end(); j++)
			{
				output << "namespace " << j->first << endl << "{ " << endl;
				vector<Uml::Class>& clss = j->second;
				for(vector<Uml::Class>::iterator jj = clss.begin(); jj != clss.end(); jj++)
				{
					string class_name = jj->name();
					//Attention - dangerous zone
					class_name = class_name.substr(0, class_name.find(Udm::cross_delimiter));
					//end of dangerous zone
					output << "\t class " << macro << " " << class_name << ";" << endl;
				};
				output << "};" << endl;

			};

			
		}
	
		output << endl << endl;
		output << "namespace " << hname << " {" << endl << endl;
		
		//don't pollute namespace
		//output << "\ttypedef Udm::Object Object;" << endl << endl;

		set<Uml::Class> classes = diagram.classes();
		set<Uml::Class> allclasses = diagram.classes();
		set<Uml::Class>::iterator c;

		for(c = classes.begin(); c != classes.end(); c++ )
		{
			Uml::Class cl = *c;
			output << "\tclass " << macro <<" "<< (string)cl.name() << ";" << endl;	
		}

		if (visitor_sup)
		{
			

			output << endl << "\tclass " << macro << " Visitor : public Udm::BaseVisitor " << endl << "\t{" << endl << "\tpublic:" <<endl;
			for(c = classes.begin(); c != classes.end(); c++ )
			{
				Uml::Class cl = *c;
				if (!cl.isAbstract())
					output << "\t\tvirtual void Visit_" << (string)cl.name() << "(const " << (string)cl.name() << "&){};" << endl;	
			}

			output << "\t\tvirtual void Visit_Object(const Udm::Object&){};" << endl;	
			output << "\t\tvirtual ~Visitor(){};" << endl;	
			output << "\t};" << endl;

		}

		output << endl << "\t " << macro << " void Initialize();" << endl;
		output << endl << "\t " << macro << " void Initialize(const Uml::Diagram &dgr);" << endl;
		output <<"\textern " << macro << " Udm::UdmDiagram diagram;" << endl << endl;


		typedef map< Uml::Class, set<Uml::Class> > cltoclsmap;
		cltoclsmap virtualchildren;
		set<Uml::Class> virtualbaseclasses;

		while(!classes.empty()) 
		{
			Uml::Class cl, cross_cl;


			
			set<Uml::Class> bases;
			for(c = classes.begin(); c != classes.end(); c++ )	
			{
				cl = *c;
				bases = cl.baseTypes();

				set<Uml::Class>::iterator i;
				for(i = bases.begin(); i != bases.end(); i++) 
				{
					if(classes.find(*i) != classes.end()) break;
				}
				if(i == bases.end()) break;  // this class had all its parents already printed
			}
			if(c == classes.end()) 
			{ // list is not empty, still we found no one to continue with
				throw udm_exception("inheritance circularity detected");
			}

			classes.erase(cl);

			if (cross_dgr)
			{
				//--//cross_cl = Uml::classByName(cross_dgr, cl.name());
				cross_cl = Uml::classByName(cross_dgr, ((string)(cl.name()) + string(Udm::cross_delimiter) + (string)diagram.name()) );
			}


//			Uml::Class tempcl(Uml::classByName(tempdiagram, cl.name()));

			output << "\tclass " << macro << " " << (string)cl.name() << " : ";

			virtualchildren.insert(cltoclsmap::value_type(cl, set<Uml::Class>()));

			if( bases.size() == 0) 
			{
// Object should be virtual, if any of the descendants has an immediate baseclass,
// that is not derived from the current class   (only descendants with >=2 baseclasses are to be checked)
				set<Uml::Class> descs = Uml::DescendantClasses(cl);
				for(set<Uml::Class>::iterator i = descs.begin(); i != descs.end(); i++) 
				{
					if(*i == cl) continue;
					set<Uml::Class> dirancs = i->baseTypes();
					if(dirancs.size() > 1) 
					{
						for(set<Uml::Class>::iterator j = dirancs.begin(); j != dirancs.end(); j++) {
							if(!Uml::IsDerivedFrom(*j, cl)) 
							{
								output << " virtual";
								virtualbaseclasses.insert(cl);
								goto bbreak;
							}
						}
					}
				}
bbreak:
				output <<  " public Udm::Object";
			}
			else 
			{
				string sep("");
				set<Uml::Class>::iterator h;
				for(h = bases.begin(); h != bases.end(); h++) 
				{
// the current base should be virtual, if any of the descendants has an immediate baseclass,
// that is derived from the current base, but not from the current class 
// (only descendants with >=2 baseclasses are to be checked)
					cltoclsmap::iterator actmapi = virtualchildren.find(*h);
					if(actmapi == virtualchildren.end()) 
					{
						throw udm_exception("internal error in class ordering");
					}
					set<Uml::Class> descs = Uml::DescendantClasses(cl);
					for(set<Uml::Class>::iterator i = descs.begin(); i != descs.end(); i++) 
					{
						if(*i == cl) continue;
						set<Uml::Class> dirancs = i->baseTypes();
						if(dirancs.size() > 1) 
						{
							for(set<Uml::Class>::iterator j = dirancs.begin(); j != dirancs.end(); j++) 
							{
								if(!Uml::IsDerivedFrom(*j, cl) && Uml::IsDerivedFrom(*j, *h)) 
								{
									
										output << sep << " virtual";
										actmapi->second.insert(cl);
										sep = "";
										goto bbreak2;
								}
							}
						}
					}

					descs = Uml::AncestorClasses(cl);
					for( i = descs.begin(); i != descs.end(); i++) 
					{
						if(*i == cl) continue;
						if (actmapi->second.find(*i) != actmapi->second.end())
						{
							
							output << sep << " virtual";
							actmapi->second.insert(cl);
							sep = "";
							goto bbreak2;
						};
					}


bbreak2:
					if(*h != cl) 
					{
						output << sep << " public " << (*h).name();
						sep = ",";
					}
				}
			}
			output << " {" << endl;
			output << "\t public:" << endl;
			output << "\t\tstatic Uml::Class meta;" << endl << endl;
			output << "\t\t" << cl.name() << "() { }" << endl;

//		D(Udm::ObjectImpl *impl) : Object(impl) { }
			output << "\t\t" << cl.name() << "(Udm::ObjectImpl *impl) : ";
			if( bases.size() == 0 ) 
			{
				output <<  "Object(impl)";
			}
			else 
			{
				string sep("");
				set<Uml::Class>::iterator i;
				for(i = bases.begin(); i != bases.end(); i++) 
				{
					output << sep << (*i).name() << "(impl)";
					sep = ",";
				}
			}

			int object_needed = 0, object_notneeded = 0;
			set<Uml::Class> ancestors = Uml::AncestorClasses(cl); 
			set<Uml::Class> additionaexplicitinitializers;

			for(set<Uml::Class>::iterator i = ancestors.begin(); i != ancestors.end(); i++) 
			{
				if(*i == cl) continue;

				if(set<Uml::Class>(i->baseTypes()).size() == 0) 
				{
					if(virtualbaseclasses.find(*i) != virtualbaseclasses.end()) object_needed++;
					else object_notneeded++;
				}
				if(bases.find(*i) != bases.end()) continue;
			
				cltoclsmap::iterator actmapi = virtualchildren.find(*i);
				set<Uml::Class> subtypes = i->subTypes();
				int found_inset = 0, found_notinset = 0;
				for(set<Uml::Class>::iterator p = subtypes.begin(); p != subtypes.end(); p++) 
				{
				
					if(!Uml::IsDerivedFrom(cl, *p)) continue;
					if(actmapi->second.find(*p) != actmapi->second.end()) found_inset++;
					else found_notinset++;
				}
				if(found_notinset > 1 || (found_inset && found_notinset))
				{
					throw udm_exception(string("Internal error resolving inheritance from class ") + string(i->name()) + " to " + string(cl.name()));
				}
				else if(found_inset) 
				{
						additionaexplicitinitializers.insert(*i);
						output << ", " << (*i).name() << "(impl)";
				}
			}
			if(object_notneeded > 1 || (object_needed && object_notneeded)) 
			{ 
					throw udm_exception("Internal error resolving inheritance from class Object to " + string(cl.name()));
			}
			else if(object_needed) 
			{
						output << ", " << "Object(impl)";
			}
			output << " { }" << endl;

//		D(const D &master) : Object(master) { }
			output << "\t\t" << cl.name() << "(const " << cl.name() << " &master) : ";
			if( bases.size() == 0 ) 
			{
				output <<  "Object(master)";
			}
			else {
				string sep("");
				set<Uml::Class>::iterator i;
				for(i = bases.begin(); i != bases.end(); i++) {
					output << sep << (*i).name() << "(master)";
					sep = ",";
				}
			}
			{
				for(set<Uml::Class>::iterator i = additionaexplicitinitializers.begin(); i != additionaexplicitinitializers.end(); i++) 
				{
						output << ", " << (*i).name() << "(master)";
				}
			}

			if(object_needed) 
			{
						output << ", " << "Object(master)";
			}
			output << " { }" << endl;

			
//		static D Cast(const Object &a) { return __Cast(a, meta); }
			output << "\t\tstatic " << cl.name() << " Cast(const Udm::Object &a) { return __Cast(a, meta); }" << endl << endl;
//		static D Create(const Object &parent, const Uml::CompositionParentRole &role) { return __Create(meta, parent, role); }
			output << "\t\tstatic " << cl.name() << " Create(const Udm::Object &parent, const Uml::CompositionChildRole &role = Udm::NULLCHILDROLE)"
							" { return __Create(meta, parent, role); }" << endl << endl;

			output << "\t\t" << cl.name() << " CreateInstance(const Udm::Object &parent, const Uml::CompositionChildRole &role = Udm::NULLCHILDROLE)"
							" { return __Create(meta, parent, role, impl); }" << endl << endl;

			output << "\t\t" << cl.name() << " CreateDerived(const Udm::Object &parent, const Uml::CompositionChildRole &role = Udm::NULLCHILDROLE)"
							" { return __Create(meta, parent, role, impl, true); }" << endl << endl;

			output << "\t\t" << "Udm::InstantiatedAttr<" << (string)cl.name() << "> Instances()"
							" { return Udm::InstantiatedAttr<" <<hname << "::" << (string)cl.name() << ">(impl);}" <<endl;

			output << "\t\t" << "template <class Pred> Udm::InstantiatedAttr<" << (string)cl.name() << ", Pred> Instances_sorted(const Pred &)"
							" { return Udm::InstantiatedAttr<" <<hname << "::" << (string)cl.name() << ", Pred>(impl);}" <<endl <<endl;


			output << "\t\t" << "Udm::DerivedAttr<" << hname << "::" << (string)cl.name() << "> Derived()"
							" { return Udm::DerivedAttr<" <<hname << "::"<< (string)cl.name() << ">(impl);}" <<endl ;

			output << "\t\t" << "template <class Pred> Udm::DerivedAttr<" << hname << "::" << (string)cl.name() << ", Pred> Derived_sorted(const Pred &)"
							" { return Udm::DerivedAttr<" <<hname << "::"<< (string)cl.name() << ", Pred>(impl);}" <<endl <<endl;


			output << "\t\t" << "Udm::ArchetypeAttr<" <<hname << "::" << (string)cl.name() << "> Archetype()"
							" { return Udm::ArchetypeAttr<" <<hname << "::" << (string)cl.name() << ">(impl);}" <<endl <<endl;

	



//		static Uml::Attribute meta_d;
//		Udm::RealAttr d() const { return Udm::RealAttr(impl, meta_d); }
			{
				set<Uml::Attribute> attributes = cl.attributes();
				for( set<Uml::Attribute>::iterator i = attributes.begin(); i != attributes.end(); i++) 
				{
					output << "\t\tstatic Uml::Attribute meta_" << (*i).name() << ";" << endl;
					Uml::Attribute k;
					string np;
					if(i->nonpersistent()) np = "Temp";
					/*
						Array support
						endre, 06/17/2002
					*/
					if ((i->max() ==1) || (i->max() == 0))
					output << "\t\tUdm::" << np << CheckAttrType(i->type()) << "Attr " << (*i).name() << 
						"() const { return Udm::" << np <<(*i).type() << "Attr(impl, meta_" << (*i).name() << "); }\n\n";
					else 
						output << "\t\tUdm::" << np << CheckAttrType(i->type()) << "AttrArr " << (*i).name() << 
						"() const { return Udm::" << np << (*i).type() << "AttrArr(impl, meta_" << (*i).name() << "); }\n\n";
						

				}
			}


//		static Uml::AssociationRole meta_src;
//		Udm::AssocAttr<A> src() const { return Udm::AssocAttr<A>(impl, meta_src); }
//		static Uml::AssociationRole meta_src;
//		Udm::PointerAttr<A> src() const { return Udm::PointerAttr<A>(impl, meta_src); }
			{
				set<Uml::AssociationRole> assocs = cl.associationRoles();
				for( set<Uml::AssociationRole>::iterator i = assocs.begin(); i != assocs.end(); i++) 
				{
				  Uml::Association pp = (*i).parent();
				  Uml::Class aclass = pp.assocClass();

				  string aname(MakeRoleName(Uml::theOther(*i)));
				  string oname(((Uml::Class)Uml::theOther(*i).target()).name());
				  if(!aclass) 
				  {
					output << "\t\tstatic Uml::AssociationRole meta_" << aname << ";" << endl;
					if(Uml::theOther(*i).isNavigable()) 
					{
						if (Uml::theOther(*i).max() == 1)
						{
							output << "\t\tUdm::PointerAttr<" << hname << "::" << oname << "> " << aname << 
							"() const { return Udm::PointerAttr<"<< hname << "::" << oname << 
							">(impl, meta_" << aname << "); }\n\n";
					
						}
						else
						{
							output << "\t\tUdm::AssocAttr<" << hname << "::" << oname << "> " << aname << 
							"() const { return Udm::AssocAttr<"<< hname << "::" << oname << 
							">(impl, meta_" << aname << "); }\n";

							output << "\t\ttemplate <class Pred> Udm::AssocAttr<" << hname << "::" << oname << ", Pred > " << aname << 
							"_sorted(const Pred &) const { return Udm::AssocAttr<"<< hname << "::" << oname << 
							", Pred>(impl, meta_" << aname << "); }\n\n";



					
						}
						/*
						string atype(Uml::theOther(*i).max() == 1 ? "Pointer" : "Assoc");
					  output << "\t\tUdm::" << atype << "Attr<" << hname << "::" << oname << "> " << aname << 
						"() const { return Udm::" << atype << "Attr<"<< hname << "::" << oname << 
						">(impl, meta_" << aname << "); }\n\n";
						*/
					}
					else 
					{
					  output << "\t\t// Access method for non-navigable association " << aname << " omitted \n\n";
					}
				  }
				  else 
				  {
					string clname(aclass.name());
					output << "\t\tstatic Uml::AssociationRole meta_" << aname << ", meta_" << aname << "_rev;" << endl;
					if(Uml::theOther(*i).isNavigable()) 
					{
					  if (Uml::theOther(*i).max() == 1)
					  {
							
							output << "\t\tUdm::AClassPointerAttr<"<< hname << "::" << clname << ", " << hname << "::" << oname << "> " << aname << 
							"() const { return Udm::AClassPointerAttr<"<< hname << "::" << clname << ", " << hname << "::" << oname << 
							">(impl, meta_" << aname << ", meta_" << aname << "_rev); }\n\n";
					  }
					  else
					  {
							
							output << "\t\tUdm::AClassAssocAttr<"<< hname << "::" << clname << ", " << hname << "::" << oname << "> " << aname << 
							"() const { return Udm::AClassAssocAttr<"<< hname << "::" << clname << ", " << hname << "::" << oname << 
							">(impl, meta_" << aname << ", meta_" << aname << "_rev); }\n";

							output << "\t\ttemplate<class Pred> Udm::AClassAssocAttr<"<< hname << "::" << clname << ", " << hname << "::" << oname << ", Pred> " << aname << 
							"_sorted(const Pred &) const { return Udm::AClassAssocAttr<"<< hname << "::" << clname << ", " << hname << "::" << oname << 
							", Pred>(impl, meta_" << aname << ", meta_" << aname << "_rev); }\n\n";
					  }
					/*	  
						  string atype(Uml::theOther(*i).max() == 1 ? "Pointer" : "Assoc");
					  output << "\t\tUdm::AClass" << atype << "Attr<"<< hname << "::" << clname << ", " << hname << "::" << oname << "> " << aname << 
						"() const { return Udm::AClass" << atype << "Attr<"<< hname << "::" << clname << ", " << hname << "::" << oname << 
						">(impl, meta_" << aname << ", meta_" << aname << "_rev); }\n\n";
						*/
					}
					else 
					{
					  output << "\t\t// Access method for non-navigable association " << aname << " omitted \n\n";
					}
				  }
				}
				if (cross_cl && (diagram != cross_dgr))
				{
					
					set<Uml::AssociationRole> assocs = cross_cl.associationRoles();
					for( set<Uml::AssociationRole>::iterator i = assocs.begin(); i != assocs.end(); i++) 
					{
						Uml::Association pp = (*i).parent();
						Uml::Class aclass = pp.assocClass();

						string aname(MakeRoleName(Uml::theOther(*i)));
						string oname(((Uml::Class)Uml::theOther(*i).target()).name());
						//Attention !!!
						oname = oname.substr(0,oname.find(Udm::cross_delimiter));
						//
						string cname(((Uml::Class)Uml::theOther(*i).target()).from());

						if(!aclass) 
						{
							output << "\t\tstatic Uml::AssociationRole meta_" << aname << ";" << endl;
							if(Uml::theOther(*i).isNavigable()) 
							{
							 
								if (Uml::theOther(*i).max() == 1 )
								{
									output << "\t\tUdm::CrossPointerAttr<" << cname << "::" << oname << "> " << aname << 
									"() const { return Udm::CrossPointerAttr<"<< cname << "::" << oname << 
									">(impl, meta_" << aname << "); }\n\n";
								}
								else
								{
									output << "\t\tUdm::CrossAssocAttr<" << cname << "::" << oname << "> " << aname << 
									"() const { return Udm::CrossAssocAttr<"<< cname << "::" << oname << 
									">(impl, meta_" << aname << "); }\n";

									output << "\t\ttemplate<class Pred> Udm::CrossAssocAttr<" << cname << "::" << oname << ", Pred> " << aname << 
									"_sorted(const Pred &) const { return Udm::CrossAssocAttr<"<< cname << "::" << oname << 
									", Pred>(impl, meta_" << aname << "); }\n\n";

								}
/*
								string atype(Uml::theOther(*i).max() == 1 ? "CrossPointer" : "CrossAssoc");
								output << "\t\tUdm::" << atype << "Attr<" << cname << "::" << oname << "> " << aname << 
								"() const { return Udm::" << atype << "Attr<"<< cname << "::" << oname << 
								">(impl, meta_" << aname << "); }\n\n";
								*/
							}
							else 
							{
								output << "\t\t// Access method for non-navigable association " << aname << " omitted \n\n";
							}
						}
						else 
						{
							//output << "\t\t// Access method for cross-package assoc class based links not yet supported" << endl;					
							string clname(aclass.name());
							//Attention !!!
							clname = clname.substr(0,clname.find(Udm::cross_delimiter));
							//end of dangerous zone
						
							string cl_dgr(aclass.from());
							output << "\t\tstatic Uml::AssociationRole meta_" << aname << ", meta_" << aname << "_rev;" << endl;
							if(Uml::theOther(*i).isNavigable()) 
							{
								if (Uml::theOther(*i).max() == 1)
								{
									output << "\t\tUdm::AClassCrossPointerAttr<"<< cl_dgr << "::" << clname << ", " << cname << "::" << oname << "> " << aname << 
									"() const { return Udm::AClassCrossPointerAttr<"<< cl_dgr << "::" << clname << ", " << cname << "::" << oname << 
									">(impl, meta_" << aname << ", meta_" << aname << "_rev); }\n\n";
								

								}
								else
								{
									output << "\t\tUdm::AClassCrossAssocAttr<"<< cl_dgr << "::" << clname << ", " << cname << "::" << oname << "> " << aname << 
									"() const { return Udm::AClassCrossAssocAttr<"<< cl_dgr << "::" << clname << ", " << cname << "::" << oname << 
									">(impl, meta_" << aname << ", meta_" << aname << "_rev); }\n";

									output << "\t\ttemplate<class Pred> Udm::AClassCrossAssocAttr<"<< cl_dgr << "::" << clname << ", " << cname << "::" << oname << ", Pred> " << aname << 
									"_sorted(const Pred &) const { return Udm::AClassCrossAssocAttr<"<< cl_dgr << "::" << clname << ", " << cname << "::" << oname << 
									", Pred>(impl, meta_" << aname << ", meta_" << aname << "_rev); }\n\n";
								
								

								}
								/*
								 string atype(Uml::theOther(*i).max() == 1 ? "CrossPointer" : "CrossAssoc");
								output << "\t\tUdm::AClass" << atype << "Attr<"<< cl_dgr << "::" << clname << ", " << cname << "::" << oname << "> " << aname << 
								"() const { return Udm::AClass" << atype << "Attr<"<< cl_dgr << "::" << clname << ", " << cname << "::" << oname << 
								">(impl, meta_" << aname << ", meta_" << aname << "_rev); }\n\n";
								*/
							}
							else 
							{
								  output << "\t\t// Access method for non-navigable association " << aname << " omitted \n\n";
							}
						}
					}
				};
			}



//		static Uml::CompositionChildRole meta_cs;
//		Udm::ChildAttr<C> cs() const { return Udm::ChildAttr<C>(impl, meta_cs); }

			{
			set<Uml::Class> childrenkinds; 

			set<Uml::CompositionParentRole> children = cl.parentRoles();
			for( set<Uml::CompositionParentRole>::iterator i = children.begin(); i != children.end(); i++) {
				Uml::CompositionParentRole k;
				
				string aname(MakeRoleName(Uml::theOther(*i)));
				Uml::Class thischildkind = (Uml::Class)Uml::theOther(*i).target();
				childrenkinds.insert(thischildkind);
				string oname(thischildkind.name());
				output << "\t\tstatic Uml::CompositionChildRole meta_" << aname << ";" << endl;
				if(Uml::theOther(*i).isNavigable()) 
				{
					if ((int)Uml::theOther(*i).max() == 1 )
					{
						output << "\t\tUdm::ChildAttr<" << hname << "::" << oname << "> " << aname << 
						"() const { return Udm::ChildAttr<" << hname << "::" << oname << 
						">(impl, meta_" << aname << "); }\n\n";
					}
					else
					{
						output << "\t\tUdm::ChildrenAttr<" << hname << "::" << oname << "> " << aname << 
						"() const { return Udm::ChildrenAttr<" << hname << "::" << oname << 
						">(impl, meta_" << aname << "); }\n";

						output << "\t\t template <class Pred> Udm::ChildrenAttr<" << hname << "::" << oname << ", Pred> " << aname << 
						"_sorted(const Pred &) const { return Udm::ChildrenAttr<" << hname << "::" << oname << 
						", Pred>(impl, meta_" << aname << "); }\n\n";
					}
					/*
					string childnum= (int)Uml::theOther(*i).max() == 1 ? "Child" : "Children";
					output << "\t\tUdm::" << childnum << "Attr<" << hname << "::" << oname << "> " << aname << 
					"() const { return Udm::" << childnum << "Attr<" << hname << "::" << oname << 
					">(impl, meta_" << aname << "); }\n\n";
						*/
				}
				else {
			      output << "\t\t// Access method for non-navigable association " << aname << " omitted \n\n";
				}
			}

			
//		Udm::ChildrenAttr<C> CChildren() const { return Udm::ChildrenAttr<C>(impl, Udm::NULLCHILDROLE); }
			for(set<Uml::Class>::iterator j = allclasses.begin(); j != allclasses.end(); j++) {
				for(set<Uml::Class>::iterator k = childrenkinds.begin(); k != childrenkinds.end(); k++) {
//cout << "Matching " << (*j).name() << " against " << (*k).name() << endl;
					if(Uml::IsDerivedFrom(*j, *k) || Uml::IsDerivedFrom(*k, *j)) 
					{
						output << "\t\tUdm::ChildrenAttr<" << hname << "::" << (*j).name() << "> " << (*j).name() << 
						"_kind_children() const { return Udm::ChildrenAttr<" << hname << "::" << (*j).name() <<
						">(impl, Udm::NULLCHILDROLE); }\n";

						output << "\t\ttemplate<class Pred> Udm::ChildrenAttr<" << hname << "::" << (*j).name() << ", Pred> " << (*j).name() << 
						"_kind_children_sorted(const Pred &) const { return Udm::ChildrenAttr<" << hname << "::" << (*j).name() <<
						", Pred>(impl, Udm::NULLCHILDROLE); }\n\n";



						break;
					}
				}
			}
			}



//		static Uml::CompositionParentRole meta_rrparent;
//		Udm::ParentAttr<D> rrparent() const { return Udm::ParentAttr<D>(impl, meta_rrparent); }


			{
				set<Uml::CompositionChildRole> children = cl.childRoles();
				for( set<Uml::CompositionChildRole>::iterator i = children.begin(); i != children.end(); i++) 
				{
					
					string aname(MakeRoleName(Uml::theOther(*i)));
					string oname(((Uml::Class)Uml::theOther(*i).target()).name());
					output << "\t\tstatic Uml::CompositionParentRole meta_" << aname << ";" << endl;
					if(Uml::theOther(*i).isNavigable()) 
					{
						output << "\t\tUdm::" << "Parent" << "Attr<" << hname << "::" << oname << "> " << aname << 
							"() const { return Udm::" << "Parent" << "Attr<" << hname << "::" << oname << 
							">(impl, meta_" << aname << "); }\n\n";
					}
					else 
					{
					  output << "\t\t// Access method for non-navigable association " << aname << " omitted \n\n";
					}
				}
			}
			bool parent_defined = false;
			set<Uml::CompositionParentRole> parroles = Uml::AncestorCompositionPeerParentRoles(cl);
			for( set<Uml::CompositionParentRole>::iterator ri = parroles.begin(); ri != parroles.end(); ri++) 
			{
				if(string(ri->name()) == "parent") parent_defined = true;
			}
			if(!parent_defined) 
			{
//		Udm::ParentAttr<Udm::Object> parent() const { return Udm::ParentAttr<Udm::Object>(impl, Udm::NULLPARENTROLE); }
//		Udm::ParentAttr<F> parent() const { return Udm::ParentAttr<F>(impl, Udm::NULLPARENTROLE); }
				set<Uml::Class> ans = Uml::CommonAncestorClasses(Uml::AncestorContainerClasses(cl));
				if(ans.empty()) 
				{
					output << "\t\tUdm::ParentAttr<Udm::Object> parent() const " << 
								"{ return Udm::ParentAttr<Udm::Object>(impl, Udm::NULLPARENTROLE); }" << endl;
				}
				else 
				{
					if (ans.size() == 1)
					{
						output << "\t\tUdm::ParentAttr<" << hname << "::" << (string)ans.begin()->name() << "> parent() const " << 
									"{ return Udm::ParentAttr<" << hname << "::" << (string)ans.begin()->name() << ">(impl, Udm::NULLPARENTROLE); }" << endl;
					}
					else
					{
						for(set<Uml::Class>::iterator ccc = ans.begin(); ccc != ans.end(); ccc++) 
						{
							output << "\t\tUdm::ParentAttr<" << hname << "::" << (string)ccc->name() << "> " << (string)ccc->name() << "_parent() const " << 
										"{ return Udm::ParentAttr<" << hname << "::" << (string)ccc->name() << ">(impl, Udm::NULLPARENTROLE); }" << endl;
						}
					}
				}
			}

//		static Uml::AssociationRole meta_End1_end_;
//		Udm::AssocEndAttr<LampX::ElectricTerminal> End1_end() const { return Udm::AssocEndAttr<LampX::ElectricTerminal>(impl, meta_End1_end_); }

			Uml::Association assoc = cl.association();
			string s = assoc.meta.name();
			if(assoc) 
			{
				set<Uml::AssociationRole> assocs = assoc.roles();
				for( set<Uml::AssociationRole>::iterator i = assocs.begin(); i != assocs.end(); i++) 
				{
					string aname(MakeRoleName(*i));
					string oname(((Uml::Class)(*i).target()).name());
					if(aname.size()) 
					{
					      output << "\t\tstatic Uml::AssociationRole meta_" << aname << "_end_;" << endl;
					      output << "\t\tUdm::AssocEndAttr<" << hname << "::" << oname << "> " << aname << 
							  "_end() const { return Udm::AssocEndAttr<"<< hname << "::" << oname << ">(impl, meta_" << aname << "_end_); }\n\n";
						
					}
				}
			}
			
			if (cross_cl && (diagram != cross_dgr))
			{
				Uml::Association cross_cl_assoc = cross_cl.association();
				string s = cross_cl_assoc.meta.name();
				
				if (cross_cl_assoc)
				{
					set<Uml::AssociationRole> assocs = cross_cl_assoc.roles();
					for( set<Uml::AssociationRole>::iterator i = assocs.begin(); i != assocs.end(); i++) 
					{
						string aname(MakeRoleName(*i));
						string oname(((Uml::Class)(*i).target()).name());

						//Attention !
						oname = oname.substr(0, oname.find(Udm::cross_delimiter));
						//end of dangerous zone 

						string from(((Uml::Class)(*i).target()).from());
						if(aname.size()) 
						{
						      output << "\t\tstatic Uml::AssociationRole meta_" << aname << "_end_;" << endl;
						      output << "\t\tUdm::CrossAssocEndAttr<" << from << "::" << oname << "> " << aname << 
								  "_end() const { return Udm::CrossAssocEndAttr<"<< from<< "::" << oname << ">(impl, meta_" << aname << "_end_); }\n\n";
						
						}
				}

				

				}
			}

			//constraints
			set<Uml::Constraint> constraints = cl.constraints();
			set<Uml::Constraint>::iterator c_i = constraints.begin();
			while (c_i != constraints.end())
			{
				output << "\t\tstatic Uml::Constraint meta_" << c_i->name() <<";" << endl;
				c_i++;
			}

			//constraint definitions
			set<Uml::ConstraintDefinition> definitions= cl.definitions();
			set<Uml::ConstraintDefinition>::iterator d_i = definitions.begin();
			while (d_i != definitions.end())
			{
				output << "\t\tstatic Uml::ConstraintDefinition meta_" << d_i->name() <<";" << endl;
				d_i++;
			}


			//visitor
			if (visitor_sup && !cl.isAbstract())
				output << "\t\tvirtual void Accept(Visitor &v){v.Visit_" << (string)cl.name() <<"(*this);};" << endl;
			output << "\t};" << endl << endl;
		}
		
		//visitor class
		
		
		output << "}\n\n#endif //" << uhname << endl;
		

}

void GenerateCPP(const Uml::Diagram &diagram,  ostream &output, string fname, const Uml::Diagram& cross_dgr = NULL, const string& macro="") 
{
		string hname = diagram.name();
		string::iterator i;
		for(i = hname.begin(); i != hname.end(); i++) {
			if(!isalnum(*i)) *i = '_';
		}

		output << "// cpp source file " << fname << ".cpp generated from diagram " << (string)diagram.name() << endl;
		output << "// generated on " << GetTime().c_str() << endl << endl;
		output << "#include \""<< fname << ".h\"" <<endl;
		output << "#include \"UmlExt.h\"" <<endl << endl; 
//		output << "#include \"UdmStatic.h\"" << endl << endl; 



		if (cross_dgr && (diagram != cross_dgr))
		{
			output << "// cross-package metainformation header file" << endl;
			output << "#include \"" << cross_dgr.name() << ".h\"" << endl << endl; 
		}

		output << "namespace " << hname << " {" << endl << endl;
//	Uml::Diagram diagram;
		output << "\tUml::Diagram umldiagram;" << endl << endl;

//	Uml::Class A::meta;
		set<Uml::Class> classes = diagram.classes();
		set<Uml::Class>::iterator c;
		for(c = classes.begin(); c != classes.end(); c++ )
		{
			Uml::Class cl = *c;
			output << "\tUml::Class " << cl.name() << "::meta;" << endl;

		}

//	Uml::Attribute A::meta_a;
		for(c = classes.begin(); c != classes.end(); c++ )	{
			Uml::Class cl = *c;
//			Uml::Class tempcl(Uml::classByName(tempdiagram, cl.name()));
			set<Uml::Attribute> attributes = cl.attributes();
/*			if(tempcl) {
				set<Uml::Attribute> tatts = tempcl.attributes();
				attributes.insert(tatts.begin(), tatts.end());
			}
			*/
			for( set<Uml::Attribute>::iterator i = attributes.begin(); i != attributes.end(); i++) {
				output << "\tUml::Attribute " << cl.name() << "::meta_" << (*i).name() << ";" << endl;
			}
		}

//	Uml::AssociationRole A::meta_dst;
		for(c = classes.begin(); c != classes.end(); c++ ) 
		{
			Uml::Class cl = *c;
			set<Uml::AssociationRole> assocs = cl.associationRoles();
			for( set<Uml::AssociationRole>::iterator i = assocs.begin(); i != assocs.end(); i++) 
			{
				Uml::Association pp = (*i).parent();
				Uml::Class aclass = pp.assocClass();
				string aname(MakeRoleName(Uml::theOther(*i)));
				if(!aclass) 
				{
					output << "\tUml::AssociationRole " << cl.name() << "::meta_" << aname << ";" << endl;
				}
				else 
				{
					output << "\tUml::AssociationRole " << cl.name() << "::meta_" << aname << ", " << cl.name() << "::meta_" << aname << "_rev;" << endl;
				}
			}

			if (cross_dgr &&(cross_dgr != diagram) )
			{
				//--//Uml::Class cross_cl = Uml::classByName(cross_dgr, cl.name());
				Uml::Class cross_cl = Uml::classByName(cross_dgr, ((string)(cl.name()) + string(Udm::cross_delimiter) + (string)(diagram.name())) );
				if(cross_cl)
				{
					set<Uml::AssociationRole> assocs = cross_cl.associationRoles();
					for( set<Uml::AssociationRole>::iterator i = assocs.begin(); i != assocs.end(); i++) 
					{
						Uml::Association pp = (*i).parent();
						Uml::Class aclass = pp.assocClass();
						string aname(MakeRoleName(Uml::theOther(*i)));
						if(!aclass) 
						{
							output << "\tUml::AssociationRole " << cl.name() << "::meta_" << aname << ";" << endl;
						}
						else 
						{
							output << "\tUml::AssociationRole " << cl.name() << "::meta_" << aname << ", " << cl.name() << "::meta_" << aname << "_rev;" << endl;
						}
					}
				};
			}

//	Uml::AssociationRole A::meta_End1_end_;

			Uml::Association assoc = cl.association();
			string s = assoc.meta.name();
			if(assoc) 
			{
				set<Uml::AssociationRole> assocs = assoc.roles();
				for( set<Uml::AssociationRole>::iterator i = assocs.begin(); i != assocs.end(); i++) 
				{
					string aname(MakeRoleName(*i));
					string oname(((Uml::Class)(*i).target()).name());
					if(aname.size()) 
					{
					      output << "\tUml::AssociationRole " << cl.name() << "::meta_" << aname << "_end_;" << endl;
					}
				}
			}
			if (cross_dgr &&(cross_dgr != diagram) )
			{
				//--//Uml::Class cross_cl = Uml::classByName(cross_dgr, cl.name());
				Uml::Class cross_cl = Uml::classByName(cross_dgr, ((string)(cl.name()) + string(Udm::cross_delimiter) + (string)(diagram.name())) );
				if(cross_cl)
				{
					Uml::Association assoc = cross_cl.association();
					string s = assoc.meta.name();
					if(assoc) 
					{
						set<Uml::AssociationRole> assocs = assoc.roles();
						for( set<Uml::AssociationRole>::iterator i = assocs.begin(); i != assocs.end(); i++) 
						{
							string aname(MakeRoleName(*i));
							string oname(((Uml::Class)(*i).target()).name());
							if(aname.size()) 
							{
								output << "\tUml::AssociationRole " << cl.name() << "::meta_" << aname << "_end_;" << endl;
							}
						}
					}
				}
			}


		}

//	Uml::CompositionParentRole A::meta_parent;
		for(c = classes.begin(); c != classes.end(); c++ ) {
			Uml::Class cl = *c;
			set<Uml::CompositionChildRole> children = cl.childRoles();
			for( set<Uml::CompositionChildRole>::iterator i = children.begin(); i != children.end(); i++) {
				string aname(MakeRoleName(Uml::theOther(*i)));
				output << "\tUml::CompositionParentRole " << cl.name() << "::meta_" << aname << ";" << endl;
			}
		}

//	Uml::CompositionChildRole D::meta_as;

		for(c = classes.begin(); c != classes.end(); c++ ) {
			Uml::Class cl = *c;
			set<Uml::CompositionParentRole> children = cl.parentRoles();
			for( set<Uml::CompositionParentRole>::iterator i = children.begin(); i != children.end(); i++) {
				string aname(MakeRoleName(Uml::theOther(*i)));
				output << "\tUml::CompositionChildRole " << cl.name() << "::meta_" << aname << ";" << endl;
			}
		}
//  Uml::Constraint D::meta_constraint
		for(c = classes.begin(); c != classes.end(); c++ ) 
		{
			Uml::Class cl = *c;
			set<Uml::Constraint> constraints = cl.constraints();
			for( set<Uml::Constraint>::iterator i = constraints.begin(); i != constraints.end(); i++) 
				output << "\tUml::Constraint " << cl.name() << "::meta_" << i->name() << ";" << endl;
		}

//	Uml::ConstraintDefinition D::meta_constraint_definition
		for(c = classes.begin(); c != classes.end(); c++ ) 
		{
			Uml::Class cl = *c;
			set<Uml::ConstraintDefinition> definitions = cl.definitions();
			for( set<Uml::ConstraintDefinition>::iterator i = definitions.begin(); i != definitions.end(); i++) 
				output << "\tUml::ConstraintDefinition " << cl.name() << "::meta_" << i->name() << ";" << endl;
		}


//	void Initialize()
//	{
//		static bool first = true;
//		if(!first) return;
//		first = false;
//		Uml::Initialize();
//		ASSERT( umldiagram == Udm::null );
//		umldiagram = Uml::CreateDiagram();
		output << "\t" << macro << " void Initialize()" << endl << "\t{" << endl;
		output << "\t\tstatic bool first = true;" << endl;
		output << "\t\tif(!first) return;" << endl;
		output << "\t\tfirst = false;" << endl;
		output << "\t\tUml::Initialize();" << endl << endl;

		if (cross_dgr && (diagram != cross_dgr))
		output << "\t\t" << cross_dgr.name() << "::Initialize();" << endl << endl;


		output << "\t\tASSERT( umldiagram == Udm::null );" << endl;
		output << "\t\tumldiagram = Uml::CreateDiagram();" << endl << endl;
		

//	A::meta = Uml::CreateClass();
		for(c = classes.begin(); c != classes.end(); c++ )
		{
			Uml::Class cl = *c;
			output << "\t\t" << cl.name() << "::meta = Uml::CreateClass();" << endl;

		}

//	A::meta_a = Uml::CreateAttribute();
		for(c = classes.begin(); c != classes.end(); c++ )	{
			Uml::Class cl = *c;
//			Uml::Class tempcl(Uml::classByName(tempdiagram, cl.name()));
			set<Uml::Attribute> attributes = cl.attributes();
/*			if(tempcl) {
				set<Uml::Attribute> tatts = tempcl.attributes();
				attributes.insert(tatts.begin(), tatts.end());
			}*/
			for( set<Uml::Attribute>::iterator i = attributes.begin(); i != attributes.end(); i++) {
				output << "\t\t" << cl.name() << "::meta_" << (*i).name() << " = Uml::CreateAttribute();" << endl;
			}
		}

//	A::meta_dst = Uml::CreateAssociationRole();
		for(c = classes.begin(); c != classes.end(); c++ ) {
				Uml::Class cl = *c;
				set<Uml::AssociationRole> assocs = cl.associationRoles();
				for( set<Uml::AssociationRole>::iterator i = assocs.begin(); i != assocs.end(); i++) {
					string aname(MakeRoleName(Uml::theOther(*i)));
					output << "\t\t" << cl.name() << "::meta_" << aname << " = Uml::CreateAssociationRole();" << endl;
			}
		}

//	A::meta_parent = Uml::CreateCompositionParentRole();
		for(c = classes.begin(); c != classes.end(); c++ ) {
			Uml::Class cl = *c;
			set<Uml::CompositionChildRole> children = cl.childRoles();
			for( set<Uml::CompositionChildRole>::iterator i = children.begin(); i != children.end(); i++) {
				string aname(MakeRoleName(Uml::theOther(*i)));
				output << "\t\t" << cl.name() << "::meta_" << aname << " = Uml::CreateCompositionParentRole();" << endl;
			}
		}

//	A::meta_as = Uml::CreateCompositionChildRole();

		for(c = classes.begin(); c != classes.end(); c++ ) {
			Uml::Class cl = *c;
			set<Uml::CompositionParentRole> children = cl.parentRoles();
			for( set<Uml::CompositionParentRole>::iterator i = children.begin(); i != children.end(); i++) {
				string aname(MakeRoleName(Uml::theOther(*i)));
				output << "\t\t" << cl.name() << "::meta_" << aname << " = Uml::CreateCompositionChildRole();" << endl;
			}
		}
// A::meta_constraintname = Uml::CreateConstraint();
		for(c = classes.begin(); c != classes.end(); c++ ) 
		{
			Uml::Class cl = *c;
			set<Uml::Constraint> constraints = cl.constraints();
			for( set<Uml::Constraint>::iterator i = constraints.begin(); i != constraints.end(); i++) 
				output << "\t\t" << cl.name() << "::meta_" << i->name() << " = Uml::CreateConstraint();" << endl;
		}


// A::meta_definitionname = Uml::CreateConstraintDefinition();
		
		for(c = classes.begin(); c != classes.end(); c++ ) 
		{
			Uml::Class cl = *c;
			set<Uml::ConstraintDefinition> definitions = cl.definitions();
			for( set<Uml::ConstraintDefinition>::iterator i = definitions.begin(); i != definitions.end(); i++) 
				output << "\t\t" << cl.name() << "::meta_" << i->name() << " = Uml::CreateConstraintDefinition();" << endl;
		}


//		Uml::InitDiagram(diagram, "Sample");
		output << "\t\tUml::InitDiagram(umldiagram, \"" << hname << "\", \"" << diagram.version() << "\");" << endl << endl;

//		Uml::InitClass(A::meta, diagram, "A", false);
//	A::meta = Uml::CreateClass();
		for(c = classes.begin(); c != classes.end(); c++ )
		{
			Uml::Class cl = *c;
			string ster;
			if(((string)cl.stereotype()).length()) ster = string(", \"")+(string)cl.stereotype()+"\"";


			string from;
			if(((string)cl.from()).length()) from = string(", \"")+(string)cl.from()+"\"";

			output << "\t\tUml::InitClass(" << cl.name() << "::meta, umldiagram, \"" <<
				cl.name() << "\", " << (cl.isAbstract() ? "true" : "false")<< ster << from  <<");" << endl;

		}
		output << "\t\t" << endl;

//		Uml::InitAttribute(A::meta_a, A::meta, "a", "String");
		for(c = classes.begin(); c != classes.end(); c++ )	
		{
			Uml::Class cl = *c;
			set<Uml::Attribute> attributes = cl.attributes();

			for( set<Uml::Attribute>::iterator i = attributes.begin(); i != attributes.end(); i++) 
			{
				//reproduce the defval string array

				if (i->defvalue())
				{
					vector<string> dva = i->defvalue();
					output << "\t\tvector<string> " << cl.name() << "_" << (*i).name() << "_dva;" <<endl;;

					vector<string>::const_iterator dva_ci = dva.begin();
	
					while (dva_ci != dva.end())
					{
						output << "\t\t" << cl.name() << "_" << (*i).name() << "_dva.push_back(\"" << *dva_ci++ << "\");" << endl; 
					};
					output << "\t\tUml::InitAttribute(" << cl.name() << "::meta_" << (*i).name() << ", " <<
						cl.name() << "::meta, \"" << (*i).name() << "\", \"" << (*i).type() << "\", " <<
						((*i).nonpersistent() ? "true" : "false") << ", " << ((*i).registry() ? "true" : "false") << ", " << (*i).min() << ", " << (*i).max() << ", " <<  
						((*i).ordered() ? "true" : "false") << ", \"" << (*i).visibility() <<
						"\", " << cl.name() << "_" << (*i).name() << "_dva" << ");" << endl;
				}
				else
				{
					
					output << "\t\tUml::InitAttribute(" << cl.name() << "::meta_" << (*i).name() << ", " <<
						cl.name() << "::meta, \"" << (*i).name() << "\", \"" << (*i).type() << "\", " <<
						((*i).nonpersistent() ? "true" : "false") << ", " << ((*i).registry() ? "true" : "false") << ", " << (*i).min() << ", " << (*i).max() << ", " <<  
						((*i).ordered() ? "true" : "false") << ", \"" << (*i).visibility() <<
						"\");" << endl;

				}
			}

			//constraints
			//(const Constraint &obj, const Class &parent, const char * name, const char * description, const char * expression);
			

			set<Uml::Constraint> constraints = cl.constraints();
			for( set<Uml::Constraint>::iterator j = constraints.begin(); j != constraints.end(); j++) 
			{
				output << "\t\tUml::InitConstraint(" << cl.name() << "::meta_" << j->name() << ", " <<
					cl.name() << "::meta, \"" << j->name() <<"\", \"" << j->description() << "\", \"" <<
					j->expression() << "\");" << endl;;
			}


			//constraint definitions
			//(const ConstraintDefinition &obj, const Class &parent, const char * name, const char * stereotype, 
			//const char * parameterList, const char * expression, const char * returnType);
			set<Uml::ConstraintDefinition> defs = cl.definitions();
			for( set<Uml::ConstraintDefinition>::iterator k = defs.begin(); k != defs.end(); k++) 
			{
				output << "\t\tUml::InitConstraintDefinition(" << cl.name() << "::meta_" << k->name() << ", " <<
					cl.name() << "::meta, \"" << k->name() <<"\", \"" << k->stereotype() << "\", \"" <<
					k->parameterList() << "\", \"" << k->expression() << "\", \"" << k->returnType() << "\");" << endl ;
			}

			//cross diagram - setting up cross-associationroles
			if (cross_dgr &&(cross_dgr != diagram))
			{
				Uml::Class cross_cl;
				Uml::Association ass;
				if (cross_dgr)
				{
					//--//cross_cl = Uml::classByName(cross_dgr, cl.name());
					cross_cl = Uml::classByName(cross_dgr, (string(cl.name()) + string(Udm::cross_delimiter) + string(diagram.name())) );
				}
				if (cross_cl)
					ass = cross_cl.association();
				if (ass)
				{
					set<Uml::AssociationRole> asrs = ass.roles();
					set<Uml::AssociationRole>::iterator asrs_i = asrs.begin();
					
					while (asrs_i != asrs.end())
					{

						output << " \t\t" << cl.name() << "::meta_" << asrs_i->name() << "_end_ = " 
							<< cross_dgr.name() << "::" << ((Uml::Class)((Uml::theOther(*asrs_i)).target())).name()<<"::meta_" << asrs_i->name() << ";" <<endl;
						asrs_i++;
					}
				};
			
			}


		}
		output << "\t\t" << endl;

		{
			const set<Uml::Association> &asss = diagram.associations();
			set<Uml::Association>::const_iterator a;
			for(a = asss.begin(); a != asss.end(); a++ )	
			{
				set<Uml::AssociationRole> ar = a->roles();
				ASSERT(ar.size() == 2);
	//	{
				output << "\t\t{" << endl;
	//		Uml::Association ass = Uml::CreateAssociation();
				output << "\t\t\tUml::Association ass = Uml::CreateAssociation();" << endl;

	//		Uml::InitAssociation(ass, diagram, "AtoB");
	// or : Uml::InitAssociationClass(ass, C::meta);
				Uml::Class aclass = a->assocClass();
				output << "\t\t\tUml::InitAssociation(ass , umldiagram, \"" << a->name() << "\");" << endl;
				if(aclass) 
				{
					output << "\t\t\tUml::InitAssociationClass(ass, " << aclass.name() << "::meta);" << endl;
				}

	//		Uml::InitAssociationRole(A::meta_dst, ass, "dst", 0, 1, B::meta);
	//								C:: meta_dst_end_ = B:meta_src_rev = A::meta_dst;
				set<Uml::AssociationRole> roles = a->roles();
				for( set<Uml::AssociationRole>::iterator i = roles.begin(); i != roles.end(); i++) 
				{
						Uml::AssociationRole zz	= Uml::theOther(*i);
						string aname(MakeRoleName(zz));
						Uml::Class cl = i->target();
						output << "\t\t\tUml::InitAssociationRole(" << cl.name() << "::meta_" << aname 
							<< ", ass, \"" << aname << "\", " <<
							(zz.isNavigable() ? "true, " : "false, ") <<
							(zz.isPrimary() ? "true, " : "false, ") <<
							zz.min() << ", " << zz.max() << ", " << ((Uml::Class)zz.target()).name() << "::meta);" << endl;
						Uml::Class aclass = a->assocClass();
						if(aclass) 
						{
							Uml::Class cl2 = zz.target();
							output << "\t\t\t" << aclass.name() << "::meta_" << aname << "_end_ = " <<
												cl2.name() << "::meta_" << MakeRoleName(*i) << "_rev = " << 
												cl.name() << "::meta_" << aname << ";" << endl;
						}
				}
	//	}
				output << "\t\t}" << endl;
			}
		}

		//cross-association role equiv. generation
		if (cross_dgr &&(cross_dgr != diagram))
		{

			//		M2::meta_src = CL::M2::meta_src;		

			for(c = classes.begin(); c != classes.end(); c++ )	
			{
				Uml::Class cl = *c;
				Uml::Class cross_cl;
				if (cross_dgr)
				{
					//--//cross_cl = Uml::classByName(cross_dgr, cl.name());
					cross_cl = Uml::classByName(cross_dgr, (string(cl.name()) + string(Udm::cross_delimiter) + string(diagram.name())) );
				}
				if (cross_cl)
				{
					set<Uml::AssociationRole> ar = cross_cl.associationRoles();
					set<Uml::AssociationRole>::iterator ar_i = ar.begin();
					while (ar_i != ar.end())
					{
						output << "\t\t"  <<cl.name() << "::meta_"<<Uml::theOther((*ar_i)).name() << 
							" = " << cross_dgr.name()<< "::" << cross_cl.name() << "::meta_"<<Uml::theOther((*ar_i)).name()<< ";" <<endl;

						Uml::AssociationRole zz	= Uml::theOther(*ar_i);
						string aname(MakeRoleName(zz));

						Uml::Association ass = ar_i->parent();
						Uml::Class a_class = ass.assocClass();
						
						if(a_class) 
						{
							Uml::Class cl2 = zz.target();
							//output << "\t\t" << a_class.from() <<"::" << a_class.name() << "::meta_" << aname << "_end_ = " <<
							//					cl2.from() << "::" << cl2.name() << "::meta_" << MakeRoleName(*ar_i) << "_rev = " << 
							//					cl.name() << "::meta_" << aname << ";" << endl;

							output << "\t\t" << cross_cl.name() << "::meta_"<<Uml::theOther((*ar_i)).name() << 
							"_rev = " << cross_dgr.name()<< "::" << cl2.name() << "::meta_"<<ar_i->name()<< ";" <<endl;

						}


						ar_i++;
					}

				}
			}
		}


		{
			const set<Uml::Composition> &comps = diagram.compositions();
			set<Uml::Composition>::const_iterator c;
			for(c = comps.begin(); c != comps.end(); c++ )	
			{
	//	{
				output << "\t\t{" << endl;
	//		Uml::Composition comp = Uml::CreateComposition();
				output << "\t\t\tUml::Composition comp = Uml::CreateComposition();" << endl;

	//		Uml::InitComposition(comp, diagram, "DtoA");
				output << "\t\t\tUml::InitComposition(comp , umldiagram, \"" << 
					c->name() << "\");" <<  endl;
				output << "\t\t" << endl;

				Uml::CompositionParentRole x1 = (*c).parentRole();
				Uml::CompositionChildRole x2 = (*c).childRole();

	//		Uml::InitCompositionParentRole(A::meta_parent, comp, "parent", D::meta);
				Uml::CompositionParentRole zzp = c->parentRole();
				Uml::CompositionChildRole zzc = c->childRole();
				string aname(MakeRoleName(zzp));
				output << "\t\t\tUml::InitCompositionParentRole(" << ((Uml::Class)zzc.target()).name() << "::meta_" << aname << 
					", comp, \""<< aname << "\", " <<
					(zzp.isNavigable() ? "true, " : "false, ") <<
					((Uml::Class)zzp.target()).name() << "::meta);" << endl;
						
	//		Uml::InitCompositionChildRole(D::meta_as, comp, "as", true, 0, -1, A::meta); 
				string cname(MakeRoleName(zzc));
				output << "\t\t\tUml::InitCompositionChildRole(" << ((Uml::Class)zzp.target()).name() << "::meta_" << cname << 
					", comp, \""<< MakeShortRoleName(zzc) << "\", " <<
						(zzc.isNavigable() ? "true, " : "false, ") <<
					zzc.min() << ", " << zzc.max() << ", " <<
					((Uml::Class)zzc.target()).name() << "::meta);" << endl;
	//	}
				output << "\t\t}" << endl;
			}
		}
		


//		Uml::AddInheritance(A::meta, C::meta);
		for(c = classes.begin(); c != classes.end(); c++ ) 
		{
			Uml::Class cl = *c;
			set<Uml::Class> st = cl.subTypes();
			for( set<Uml::Class>::iterator i = st.begin(); i != st.end(); i++) 
			{
				output << "\t\tUml::AddInheritance(" << cl.name() << "::meta, " << (*i).name() << "::meta);" << endl;
			}
		}			
		output << "\t\t" << endl << "\t}" << endl;
//	Udm::UdmDiagram diagram = { &umldiagram, Initialize };
		output << "\t" << macro << " Udm::UdmDiagram diagram = { &umldiagram, Initialize };" << endl;

		output << "\tstatic struct _regClass" << endl;
		output << "\t{" << endl;
		output << "\t\t_regClass()" << endl;
		output << "\t\t{" << endl;
		output << "\t\t\tUdm::MetaDepository::StoreDiagram(\"" << diagram.name() <<"\", diagram);" << endl;
		output << "\t\t};" << endl;
		output << "\t\t~_regClass()" << endl;

		output << "\t\t{" << endl;
		output <<"\t\t\tUdm::MetaDepository::RemoveDiagram(\"" << diagram.name() << "\");" << endl;
		output <<"\t\t};" <<endl;
		output <<"\t} __regUnUsed;" << endl << endl;


		output << "}" << endl <<"// END " << hname << ".cpp" <<endl;
}

// --------------------------- NEW CPP FORMAT Generation

void GenerateCompositionInit(const Uml::Composition &c, ostream output, int funct_no)
{
	output << "\tvoid InitComposition" << funct_no << "()" << endl << "\t{" << endl;


	output << "\t\tUml::Composition comp = Uml::Composition::Create(umldiagram);" << endl;

	if (!((string)c.name()).empty())output << "\t\t\tcomp.name() =\"" << c.name() << "\";" <<endl;
	output << "\t\tcomp.nonpersistent() =" << (c.nonpersistent() ? "true" : "false") << ";" << endl;

				

	Uml::CompositionParentRole x1 = c.parentRole();
	Uml::CompositionChildRole x2 = c.childRole();

	Uml::CompositionParentRole zzp = c.parentRole();
	Uml::CompositionChildRole zzc = c.childRole();
	string aname(MakeRoleName(zzp));
	output << "\t\t" << ((Uml::Class)zzc.target()).name() << "::meta_" << aname <<" = Uml::CompositionParentRole::Create(comp);" << endl;
	output << "\t\t" << ((Uml::Class)zzc.target()).name() << "::meta_" << aname <<".name() = \"" << aname << "\";" << endl;
	output << "\t\t" << ((Uml::Class)zzc.target()).name() << "::meta_" << aname <<".isNavigable() = " << (zzp.isNavigable() ? "true;" : "false;") << endl;
	output << "\t\t" << ((Uml::Class)zzc.target()).name() << "::meta_" << aname <<".target() = " << ((Uml::Class)zzp.target()).name() << "::meta;" << endl;

	string cname(MakeRoleName(zzc));

	output << "\t\t" << ((Uml::Class)zzp.target()).name() << "::meta_" << cname << " = Uml::CompositionChildRole::Create(comp);" << endl;

	output << "\t\t" << ((Uml::Class)zzp.target()).name() << "::meta_" << cname << ".name() = \"" << MakeShortRoleName(zzc) << "\";" << endl;
	output << "\t\t" << ((Uml::Class)zzp.target()).name() << "::meta_" << cname << ".min() = " << zzc.min() << ";"<< endl;
	output << "\t\t" << ((Uml::Class)zzp.target()).name() << "::meta_" << cname << ".max() = " << zzc.max() << ";"<< endl;
	output << "\t\t" << ((Uml::Class)zzp.target()).name() << "::meta_" << cname << ".isNavigable() = " << (zzc.isNavigable() ? "true;" : "false;") << endl;
	output << "\t\t" << ((Uml::Class)zzp.target()).name() << "::meta_" << cname << ".target() = " << ((Uml::Class)zzc.target()).name() << "::meta;" << endl;

	output << "\t}" << endl;
};

void GenerateAssociationInit(const Uml::Association &a,  ostream &output, int funct_no)
{
	output << "\tvoid InitAssociation" << funct_no << "()" << endl << "\t{" << endl;

	set<Uml::AssociationRole> ar = a.roles();
	ASSERT(ar.size() == 2);
	
	output << "\t\tUml::Association ass = Uml::Association::Create(umldiagram);" << endl;

	Uml::Class aclass = a.assocClass();
	if (!((string)a.name()).empty())output << "\t\tass.name() = \"" << a.name() << "\";" <<endl;
	if(aclass) 
	{
		output << "\t\tass.assocClass() = " << aclass.name() << "::meta;" << endl;
	}

	set<Uml::AssociationRole> roles = a.roles();
	for( set<Uml::AssociationRole>::iterator i = roles.begin(); i != roles.end(); i++) 
	{
		Uml::AssociationRole zz	= Uml::theOther(*i);
		string aname(MakeRoleName(zz));
		Uml::Class cl = i->target();
		
		output << "\t\t" << cl.name() << "::meta_" << aname << " = Uml::AssociationRole::Create(ass);" <<endl;
		output << "\t\t" << cl.name() << "::meta_" << aname << ".name() = \"" << aname << "\";" <<endl;
		output << "\t\t" << cl.name() << "::meta_" << aname << ".min() = " << zz.min() << ";" <<endl;
		output << "\t\t" << cl.name() << "::meta_" << aname << ".max() = " << zz.max() << ";" <<endl;
		output << "\t\t" << cl.name() << "::meta_" << aname << ".isNavigable() = " << (zz.isNavigable() ? "true":"false") << ";" <<endl;
		output << "\t\t" << cl.name() << "::meta_" << aname << ".isPrimary() = " << (zz.isPrimary() ? "true":"false") << ";" <<endl;
		output << "\t\t" << cl.name() << "::meta_" << aname << ".target() = " << ((Uml::Class)zz.target()).name() << "::meta;" <<endl;

		Uml::Class aclass = a.assocClass();
		if(aclass) 
		{
			Uml::Class cl2 = zz.target();
			output << "\t\t" << aclass.name() << "::meta_" << aname << "_end_ = " <<
							cl2.name() << "::meta_" << MakeRoleName(*i) << "_rev = " << 
							cl.name() << "::meta_" << aname << ";" << endl;
		}
	}
	
	
	output << "\t};" << endl;

}

void GenerateNewCPP(const Uml::Diagram &diagram,  ostream &output, string fname, const Uml::Diagram& cross_dgr = NULL, const string& macro = "") 
{
		string hname = diagram.name();
		string::iterator i;
		for(i = hname.begin(); i != hname.end(); i++) 
		{
			if(!isalnum(*i)) *i = '_';
		}

		output << "// cpp(meta datanetwork format) source file " << fname << ".cpp generated from diagram " << (string)diagram.name() << endl;
		output << "// generated on " << GetTime().c_str() << endl << endl;
		output << "#include \""<< fname << ".h\"" <<endl;
		output << "#include \"UmlExt.h\"" <<endl << endl; 
		output << "#include \"UdmStatic.h\"" << endl << endl; 

		if (cross_dgr && (diagram != cross_dgr))
		{
			output << "// cross-package metainformation header file" << endl;
			output << "#include \"" << cross_dgr.name() << ".h\"" << endl << endl; 
		}


		output << "namespace " << hname << " {" << endl << endl;
//	Uml::Diagram diagram;
		output << "\tUml::Diagram umldiagram;" << endl << endl;

//	Uml::Class A::meta;
		set<Uml::Class> classes = diagram.classes();
		set<Uml::Class>::iterator c;
		for(c = classes.begin(); c != classes.end(); c++ )
		{
			Uml::Class cl = *c;
			output << "\tUml::Class " << cl.name() << "::meta;" << endl;

		}

//	Uml::Attribute A::meta_a;
		for(c = classes.begin(); c != classes.end(); c++ )	
		{
			Uml::Class cl = *c;
			//Uml::Class tempcl(Uml::classByName(tempdiagram, cl.name()));
			set<Uml::Attribute> attributes = cl.attributes();
			for( set<Uml::Attribute>::iterator i = attributes.begin(); i != attributes.end(); i++) 
			{
				output << "\tUml::Attribute " << cl.name() << "::meta_" << (*i).name() << ";" << endl;
			}
		}

//declarations for associationroles
//	Uml::AssociationRole A::meta_dst;
		for(c = classes.begin(); c != classes.end(); c++ ) 
		{
			Uml::Class cl = *c;

			
			set<Uml::AssociationRole> assocs = cl.associationRoles();
			for( set<Uml::AssociationRole>::iterator i = assocs.begin(); i != assocs.end(); i++) 
			{
				Uml::Association pp = (*i).parent();
				Uml::Class aclass = pp.assocClass();
				string aname(MakeRoleName(Uml::theOther(*i)));
				if(!aclass) 
				{
					output << "\tUml::AssociationRole " << cl.name() << "::meta_" << aname << ";" << endl;
				}
				else 
				{
					output << "\tUml::AssociationRole " << cl.name() << "::meta_" << aname << ", " << cl.name() << "::meta_" << aname << "_rev;" << endl;
				}
			}

			/*
				Here is printed the decalaration of the cross-association roles
			*/
			if (cross_dgr &&(cross_dgr != diagram) )
			{
				//--//Uml::Class cross_cl = Uml::classByName(cross_dgr, cl.name());
				string cross_cl_name = string(cl.name()) + string(Udm::cross_delimiter) + string(diagram.name());
				Uml::Class cross_cl = Uml::classByName(cross_dgr, cross_cl_name );
				if(cross_cl)
				{
					set<Uml::AssociationRole> assocs = cross_cl.associationRoles();
					for( set<Uml::AssociationRole>::iterator i = assocs.begin(); i != assocs.end(); i++) 
					{
						Uml::Association pp = (*i).parent();
						Uml::Class aclass = pp.assocClass();
						string aname(MakeRoleName(Uml::theOther(*i)));
						if(!aclass) 
						{
							output << "\tUml::AssociationRole " << cl.name() << "::meta_" << aname << ";" << endl;
						}
						else 
						{
							output << "\tUml::AssociationRole " << cl.name() << "::meta_" << aname << ", " << cl.name() << "::meta_" << aname << "_rev;" << endl;
						}
					}
				};
			};
//	Uml::AssociationRole A::meta_End1_end_;

			Uml::Association assoc = cl.association();
			string s = assoc.meta.name();
			if(assoc) 
			{
				set<Uml::AssociationRole> assocs = assoc.roles();
				for( set<Uml::AssociationRole>::iterator i = assocs.begin(); i != assocs.end(); i++) 
				{
					string aname(MakeRoleName(*i));
					string oname(((Uml::Class)(*i).target()).name());
					if(aname.size()) 
					{
					  output << "\tUml::AssociationRole " << cl.name() << "::meta_" << aname << "_end_;" << endl;
					}
				}
			};

			if (cross_dgr &&(cross_dgr != diagram) )
			{
				//--//Uml::Class cross_cl = Uml::classByName(cross_dgr, cl.name());
				Uml::Class cross_cl = Uml::classByName(cross_dgr, (string(cl.name()) + string(Udm::cross_delimiter) + string(diagram.name())) );
				if(cross_cl)
				{
					Uml::Association assoc = cross_cl.association();
					string s = assoc.meta.name();
					if(assoc) 
					{
						set<Uml::AssociationRole> assocs = assoc.roles();
						for( set<Uml::AssociationRole>::iterator i = assocs.begin(); i != assocs.end(); i++) 
						{
							string aname(MakeRoleName(*i));
							string oname(((Uml::Class)(*i).target()).name());
							if(aname.size()) 
							{
								output << "\tUml::AssociationRole " << cl.name() << "::meta_" << aname << "_end_;" << endl;
							}
						}
					}
				}
			};
		};//end for cycle for declaration for associationRoles

//	Uml::CompositionParentRole A::meta_parent;
		for(c = classes.begin(); c != classes.end(); c++ ) 
		{
			Uml::Class cl = *c;
			set<Uml::CompositionChildRole> children = cl.childRoles();
			for( set<Uml::CompositionChildRole>::iterator i = children.begin(); i != children.end(); i++) {
				string aname(MakeRoleName(Uml::theOther(*i)));
				output << "\tUml::CompositionParentRole " << cl.name() << "::meta_" << aname << ";" << endl;
			}
		}

//	Uml::CompositionChildRole D::meta_as;

		for(c = classes.begin(); c != classes.end(); c++ ) 
		{
			Uml::Class cl = *c;
			set<Uml::CompositionParentRole> children = cl.parentRoles();
			for( set<Uml::CompositionParentRole>::iterator i = children.begin(); i != children.end(); i++) {
				string aname(MakeRoleName(Uml::theOther(*i)));
				output << "\tUml::CompositionChildRole " << cl.name() << "::meta_" << aname << ";" << endl;
			}
		}

//  Uml::Constraint D::meta_constraint
		for(c = classes.begin(); c != classes.end(); c++ ) 
		{
			Uml::Class cl = *c;
			set<Uml::Constraint> constraints = cl.constraints();
			for( set<Uml::Constraint>::iterator i = constraints.begin(); i != constraints.end(); i++) 
				output << "\tUml::Constraint " << cl.name() << "::meta_" << i->name() << ";" << endl;
		}


//	Uml::ConstraintDefinition D::meta_constraint_definition
		for(c = classes.begin(); c != classes.end(); c++ ) 
		{
			Uml::Class cl = *c;
			set<Uml::ConstraintDefinition> definitions = cl.definitions();
			for( set<Uml::ConstraintDefinition>::iterator i = definitions.begin(); i != definitions.end(); i++) 
				output << "\tUml::ConstraintDefinition " << cl.name() << "::meta_" << i->name() << ";" << endl;
		}


//void Create()
// {
		output << "\tvoid Creates()" << endl << "\t{" << endl;
		
//	A::meta = Uml::CreateClass();
		for(c = classes.begin(); c != classes.end(); c++ )
		{
			Uml::Class cl = *c;
			output << "\t\t" << cl.name() << "::meta = Uml::Class::Create(umldiagram);" << endl;

		}

//	A::meta_a = Uml::CreateAttribute();
		for(c = classes.begin(); c != classes.end(); c++ )	
		{
			Uml::Class cl = *c;
			
			set<Uml::Attribute> attributes = cl.attributes();
			for( set<Uml::Attribute>::iterator i = attributes.begin(); i != attributes.end(); i++) 
			{
				output << "\t\t" << cl.name() << "::meta_" << (*i).name() << " = Uml::Attribute::Create(" << cl.name()<<"::meta);" << endl;
			}

			//constraints
			set<Uml::Constraint> constraints = cl.constraints();
			for (set<Uml::Constraint>::iterator j = constraints.begin(); j != constraints.end(); j++)
			{
				output << "\t\t" << cl.name() << "::meta_" << j->name() << " = Uml::Constraint::Create(" << cl.name() << "::meta);" << endl;
			}

			//constraintdefinitions
			set<Uml::ConstraintDefinition> definitions = cl.definitions();
			for (set<Uml::ConstraintDefinition>::iterator k = definitions.begin(); k != definitions.end(); k++)
			{
				output << "\t\t" << cl.name() << "::meta_" << k->name() << " = Uml::ConstraintDefinition::Create(" << cl.name() << "::meta);" << endl;
			}
		}


		output << "\t}" << endl;
//void InitAttributes()
// {
		output << "\tvoid InitClassesAttributes()" << endl << "\t{" << endl;

			for(c = classes.begin(); c != classes.end(); c++ )
		{
			Uml::Class cl = *c;
			string ster;
			if(((string)cl.stereotype()).length()) ster = string(", \"")+(string)cl.stereotype()+"\"";
			//output << "\t\tUml::InitClass(" << cl.name() << "::meta, umldiagram, \"" <<
			//	cl.name() << "\", " << (cl.isAbstract() ? "true" : "false")<< ster <<");" << endl;
			output << "\t\t" << cl.name() << "::meta.name() = \"" << cl.name() << "\";" <<endl;
			output << "\t\t" << cl.name() << "::meta.isAbstract() = " << (cl.isAbstract() ? "true" : "false") << ";" << endl;
			if (!ster.empty()) 
				output << "\t\t" << cl.name() << "::meta.stereotype() = \"" << cl.stereotype() << "\";" <<endl;

			string from = cl.from();
			if (!from.empty())
				output << "\t\t" << cl.name() << "::meta.from() = \"" << cl.from() << "\";" <<endl;



		}
		output << "\t\t" << endl;

//		Uml::InitAttribute(A::meta_a, A::meta, "a", "String");
		for(c = classes.begin(); c != classes.end(); c++ )	
		{
			Uml::Class cl = *c;

			
			set<Uml::Attribute> attributes = cl.attributes();

			for( set<Uml::Attribute>::iterator i = attributes.begin(); i != attributes.end(); i++) 
			{
			
				//	output << "\t\tUml::InitAttribute(" << cl.name() << "::meta_" << (*i).name() << ", " <<
				//		cl.name() << "::meta, \"" << (*i).name() << "\", \"" << (*i).type() << "\");" << endl;

				if (i->defvalue())
				{
					vector<string> dva = i->defvalue();
					output << "\t\tvector<string> " << cl.name() << "_" << (*i).name() << "_dva;" <<endl;
					vector<string>::const_iterator dva_ci = dva.begin();
					while (dva_ci != dva.end())
					{
						output << "\t\t" << cl.name() << "_" <<(*i).name() << "_dva.push_back(\"" << *dva_ci++ << "\");" << endl; 
					};
				}
				output << "\t\t" << cl.name() << "::meta_" << (*i).name() << ".name() = \"" << (*i).name() << "\";" << endl;
				output << "\t\t" << cl.name() << "::meta_" << (*i).name() << ".type() = \"" << (*i).type() << "\";" << endl;
				output << "\t\t" << cl.name() << "::meta_" << (*i).name() << ".min() = " << (*i).min() << ";" << endl;
				output << "\t\t" << cl.name() << "::meta_" << (*i).name() << ".max() = " << (*i).max() << ";" << endl;
				output << "\t\t" << cl.name() << "::meta_" << (*i).name() << ".nonpersistent() = " << ((*i).nonpersistent() ? "true" : "false") << ";" << endl;
				output << "\t\t" << cl.name() << "::meta_" << (*i).name() << ".registry() = " << ((*i).registry() ? "true" : "false") << ";" << endl;
				output << "\t\t" << cl.name() << "::meta_" << (*i).name() << ".ordered() = " << ((*i).ordered() ? "true" : "false") << ";" << endl;
				output << "\t\t" << cl.name() << "::meta_" << (*i).name() << ".visibility() = \"" << (*i).visibility() << "\";" << endl;
				if (i->defvalue())
					output << "\t\t" << cl.name() << "::meta_" << (*i).name() << ".defvalue() = " << cl.name() << "_" <<(*i).name() << "_dva;" << endl;
				else
					output << "\t\t" << cl.name() << "::meta_" << (*i).name() << ".defvalue() = vector<string>();" << endl;
			}

			set<Uml::Constraint> constraints= cl.constraints();
			for( set<Uml::Constraint>::iterator j = constraints.begin(); j != constraints.end(); j++) 
			{
				output << "\t\t" << cl.name() << "::meta_" << j->name() << ".name() = \"" << j->name() << "\";" << endl;
				output << "\t\t" << cl.name() << "::meta_" << j->name() << ".description() = \"" << j->description() << "\";" << endl;
				output << "\t\t" << cl.name() << "::meta_" << j->name() << ".expression() = \"" << j->expression() << "\";" << endl;
			}
		

			set<Uml::ConstraintDefinition> defs = cl.definitions();
			for( set<Uml::ConstraintDefinition>::iterator k = defs.begin(); k != defs.end(); k++) 
			{
				output << "\t\t" << cl.name() << "::meta_" << k->name() << ".name() = \"" << k->name() << "\";" << endl;
				output << "\t\t" << cl.name() << "::meta_" << k->name() << ".stereotype() = \"" << k->stereotype() << "\";" << endl;
				output << "\t\t" << cl.name() << "::meta_" << k->name() << ".parameterList() = \"" << k->parameterList() << "\";" << endl;
				output << "\t\t" << cl.name() << "::meta_" << k->name() << ".expression() = \"" << k->expression() << "\";" << endl;
				output << "\t\t" << cl.name() << "::meta_" << k->name() << ".returnType() = \"" << k->returnType() << "\";" << endl;
				
			}


			if (cross_dgr &&(cross_dgr != diagram))
			{
				Uml::Class cross_cl;
				Uml::Association ass;
				if (cross_dgr)
				{
					//--//cross_cl = Uml::classByName(cross_dgr, cl.name());
					cross_cl = Uml::classByName(cross_dgr, (string(cl.name()) + string(Udm::cross_delimiter) + string(diagram.name())) );
				}
				if (cross_cl)
					ass = cross_cl.association();
				if (ass)
				{
					set<Uml::AssociationRole> asrs = ass.roles();
					set<Uml::AssociationRole>::iterator asrs_i = asrs.begin();
					
					while (asrs_i != asrs.end())
					{

						output << " \t\t" << cl.name() << "::meta_" << asrs_i->name() << "_end_ = " 
							<< cross_dgr.name() << "::" << ((Uml::Class)((Uml::theOther(*asrs_i)).target())).name()<<"::meta_" << asrs_i->name() << ";" <<endl;
						asrs_i++;
					}
				};
			
			}



		}
		output << "\t\t" << endl;


		output << "\t}" << endl;
	
		int ass_inits = 0;
		{
			const set<Uml::Association> &asss = diagram.associations();
			set<Uml::Association>::const_iterator a;
			for(a = asss.begin(); a != asss.end(); a++ )	
				GenerateAssociationInit(*a, output, ass_inits++);
		}	
			

		int comp_inits = 0;
		{
			const set<Uml::Composition> &comps = diagram.compositions();
			set<Uml::Composition>::const_iterator c;
			for(c = comps.begin(); c != comps.end(); c++ )	
				GenerateCompositionInit(*c, output, comp_inits++);
		}
		

//	void Initialize()
//	{
//		static bool first = true;
//		if(!first) return;
//		first = false;
//		Uml::Initialize();
//		ASSERT( umldiagram == Udm::null );
//		umldiagram = Uml::CreateDiagram();
/*
		UdmStatic::StaticDataNetwork meta_dn(Uml::diagram);
		meta_dn.CreateNew("LampDiagram.mem", "", RootFolder::meta);

		ASSERT( umldiagram == Udm::null );
		
		//umldiagram = Uml::CreateDiagram();
		umldiagram = Uml::Diagram::Cast(meta_dn.GetRootObject());
*/
		output << "\t" << macro << " void Initialize()" << endl << "\t{" << endl;
		output << "\t\tstatic bool first = true;" << endl;
		output << "\t\tif(!first) return;" << endl;
		output << "\t\tfirst = false;" << endl;

		output << "\t\tUml::Initialize();" << endl << endl;

		if (cross_dgr && (diagram != cross_dgr))
			output << "\t\t" << cross_dgr.name() << "::Initialize();" << endl << endl;

		output << "\t\tASSERT( umldiagram == Udm::null );" << endl;
	//	output << "\t\tumldiagram = Uml::CreateDiagram();" << endl << endl;
		output << "\t\tUdmStatic::StaticDataNetwork * meta_dn = new UdmStatic::StaticDataNetwork(Uml::diagram);" << endl;
		output << "\t\tmeta_dn->CreateNew(\"" << hname << ".mem\", \"\", Uml::Diagram::meta, Udm::CHANGES_LOST_DEFAULT);"<< endl;
		output << "\t\tumldiagram = Uml::Diagram::Cast(meta_dn->GetRootObject());" << endl;
		output << "\t\tumldiagram.name() =\"" << hname << "\";" << endl;
		output << "\t\tumldiagram.version() =\"" << diagram.version() << "\";" << endl;

		output << "\t\tCreates();" << endl;
		output << "\t\tInitClassesAttributes();" << endl;
		

//		Uml::InitDiagram(diagram, "Sample");
		output << "\t\tUml::InitDiagram(umldiagram, \"" << hname << "\", \"" << diagram.version() << "\");" << endl << endl;
		

//		Uml::InitClass(A::meta, diagram, "A", false);
//	A::meta = Uml::CreateClass();
	
		{

			for (int ass_inits_i = 0; ass_inits_i < ass_inits; ass_inits_i++)
				output << "\t\tInitAssociation" << ass_inits_i << "();" << endl;
			
		}
		if (cross_dgr &&(cross_dgr != diagram))
		{

			//		M2::meta_src = CL::M2::meta_src;		

			for(c = classes.begin(); c != classes.end(); c++ )	
			{
				Uml::Class cl = *c;
				Uml::Class cross_cl;
				if (cross_dgr)
				{
					cross_cl = Uml::classByName(cross_dgr, (string(cl.name()) + string(Udm::cross_delimiter) + string(diagram.name())) );
					//--//cross_cl = Uml::classByName(cross_dgr, cl.name());
				}
				if (cross_cl)
				{
					set<Uml::AssociationRole> ar = cross_cl.associationRoles();
					set<Uml::AssociationRole>::iterator ar_i = ar.begin();
					while (ar_i != ar.end())
					{
						//--//output << "\t\t" << cross_cl.name() << "::meta_"<<Uml::theOther((*ar_i)).name() << 
						//--//	" = " << cross_dgr.name()<< "::" << cross_cl.name() << "::meta_"<<Uml::theOther((*ar_i)).name()<< ";" <<endl;

						output << "\t\t" << cl.name() << "::meta_"<<Uml::theOther((*ar_i)).name() << 
						" = " << cross_dgr.name()<< "::" << cross_cl.name() << "::meta_"<<Uml::theOther((*ar_i)).name()<< ";" <<endl;

						Uml::AssociationRole zz	= Uml::theOther(*ar_i);
						string aname(MakeRoleName(zz));

						Uml::Association ass = ar_i->parent();
						Uml::Class a_class = ass.assocClass();
						
						if(a_class) 
						{
							Uml::Class cl2 = zz.target();
							//output << "\t\t" << a_class.from() <<"::" << a_class.name() << "::meta_" << aname << "_end_ = " <<
							//					cl2.from() << "::" << cl2.name() << "::meta_" << MakeRoleName(*ar_i) << "_rev = " << 
							//					cl.name() << "::meta_" << aname << ";" << endl;

							//--//output << "\t\t" << cross_cl.name() << "::meta_"<<Uml::theOther((*ar_i)).name() << 
							//--//"_rev = " << cross_dgr.name()<< "::" << cl2.name() << "::meta_"<<ar_i->name()<< ";" <<endl;

							output << "\t\t" << cl.name() << "::meta_"<<Uml::theOther((*ar_i)).name() << 
							"_rev = " << cross_dgr.name()<< "::" << cl2.name() << "::meta_"<<ar_i->name()<< ";" <<endl;

						}


						ar_i++;
					}

				}
			}
		}

		{
			for (int  comp_inits_i = 0; comp_inits_i < comp_inits; comp_inits_i++)
				output << "\t\tInitComposition" << comp_inits_i << "();" << endl;
		}



//		Uml::AddInheritance(A::meta, C::meta);
		for(c = classes.begin(); c != classes.end(); c++ ) {
			Uml::Class cl = *c;
			set<Uml::Class> st = cl.subTypes();
			for( set<Uml::Class>::iterator i = st.begin(); i != st.end(); i++) {
			//	output << "\t\tUml::AddInheritance(" << cl.name() << "::meta, " << (*i).name() << "::meta);" << endl;
				output << "\t\t" << cl.name() << "::meta.subTypes() +=  " << (*i).name() << "::meta;" << endl;
			}
		}	

		output << "\t\t" << endl << "\t}" << endl;	//end of Initialize()

// Initialize(const Uml::Diagram &dgr);
		

		output << "\tvoid Initialize(const Uml::Diagram & dgr)" << endl << "\t{" << endl;

		output << "\t\t" << "umldiagram = dgr;" << endl;
		for (c = classes.begin(); c != classes.end(); c++)
		{
			Uml::Class cl = *c;
			output << "\t\tUml::SetClass(" << cl.name() <<"::meta, umldiagram, \"" << cl.name() << "\");" <<endl;

			//attributes
			set<Uml::Attribute> cl_atts = cl.attributes();
			for (set<Uml::Attribute>::iterator cla_i = cl_atts.begin(); cla_i != cl_atts.end(); cla_i++)
			{
				Uml::Attribute a = *cla_i;
				output << "\t\tUml::SetAttribute(" << cl.name() <<"::meta_" << a.name() << "," << cl.name() << "::meta, \"" << a.name() << "\");" <<endl;

			};
		};

		output << "\t\t" << "//composition child roles " << endl;
		for (c = classes.begin(); c != classes.end(); c++)
		{
		
			Uml::Class cl = *c;

			//composition parent roles
			set<Uml::CompositionChildRole> ccrs = cl.childRoles();
			for (set<Uml::CompositionChildRole>::iterator ccrs_i = ccrs.begin(); ccrs_i != ccrs.end(); ccrs_i ++)
			{
				Uml::CompositionChildRole ccr = *ccrs_i;
				string rolename = Uml::MakeRoleName(Uml::theOther(ccr));
				string theot_name = ((Uml::Class)Uml::theOther(ccr).target()).name();
				output << "\t\tUml::SetParentRole(" << cl.name() <<"::meta_" << rolename << "," << cl.name() << "::meta," << theot_name << "::meta, \"" << ccr.name() << "\", \"" << Uml::theOther(ccr).name() << "\");" <<endl;

			};

		};
		
		output << "\t\t" << "// composition parentroles " << endl;
		for (c = classes.begin(); c != classes.end(); c++)
		{
			Uml::Class cl = *c;

			//composition child roles
			set<Uml::CompositionParentRole> cprs = cl.parentRoles();
			for (set<Uml::CompositionParentRole>::iterator cprs_i = cprs.begin(); cprs_i != cprs.end(); cprs_i ++)
			{
				Uml::CompositionParentRole cpr = *cprs_i;
				string rolename = Uml::MakeRoleName(Uml::theOther(cpr));
				string theot_name = ((Uml::Class)Uml::theOther(cpr).target()).name();
				output << "\t\tUml::SetChildRole(" << cl.name() <<"::meta_" << rolename << "," << cl.name() << "::meta," << theot_name << "::meta, \"" << cpr.name() << "\", \"" << Uml::theOther(cpr).name() << "\");" <<endl;

			};
		};
		
		output << "\t\t" << "// Association roles " << endl;
		for (c = classes.begin(); c != classes.end(); c++)
		{
			Uml::Class cl = *c;

			//association roles
			set<Uml::AssociationRole> ars = cl.associationRoles();
			for (set<Uml::AssociationRole>::iterator ars_i = ars.begin(); ars_i != ars.end(); ars_i ++)
			{
				Uml::AssociationRole ar = *ars_i;
				string rolename = Uml::MakeRoleName(Uml::theOther(ar));
				string theot_name = ((Uml::Class)Uml::theOther(ar).target()).name();


				//output << "\t\tUml::SetAssocRole(" << cl.name() <<"::meta_" << rolename << "," << ((Uml::Class)Uml::theOther(ar).target()).name() << "::meta," << theot_name << "::meta, \"" << ar.name() << "\");" <<endl;
				output << "\t\tUml::SetAssocRole(" << cl.name() <<"::meta_" << rolename << "," << cl.name() << "::meta," << theot_name << "::meta, \"" << ar.name() << "\");" <<endl;

				Uml::Association as = ar.parent();
				Uml::Class aclass = as.assocClass();
				if (aclass)
				{
					string revrolename = Uml::MakeRoleName(ar) + "_rev";
					//set the _rev -s
					output << "\t\t" << ((Uml::Class)Uml::theOther(ar).target()).name() << "::meta_" << revrolename <<" = " << cl.name() << "::meta_" << rolename <<";" << endl;
				}

			};

			//_end_ association roles
			Uml::Association as = cl.association();
			if (as)
			{
				set<Uml::AssociationRole> ars = as.roles();
				for (set<Uml::AssociationRole>::iterator ars_i = ars.begin();ars_i != ars.end(); ars_i++)
				{
					Uml::AssociationRole ar = *ars_i;
					output << "\t\t" << cl.name() << "::meta_" << Uml::MakeRoleName(ar) << "_end_ = " << ((Uml::Class)Uml::theOther(ar).target()).name() << "::meta_" <<Uml::MakeRoleName(ar) << ";" << endl;

				};
			};


			output << endl;
		};


		
		output << "\t\t" << endl << "\t}" << endl; //end of Initialize(const Uml::Diagram & dgr);
//	Udm::UdmDiagram diagram = { &umldiagram, Initialize };
		output << "\t" << macro << " Udm::UdmDiagram diagram = { &umldiagram, Initialize };" << endl;

		output << "\tstatic struct _regClass" << endl;
		output << "\t{" << endl;
		output << "\t\t_regClass()" << endl;
		output << "\t\t{" << endl;
		output << "\t\t\tUdm::MetaDepository::StoreDiagram(\"" << diagram.name() <<"\", diagram);" << endl;
		output << "\t\t};" << endl;
		output << "\t\t~_regClass()" << endl;

		output << "\t\t{" << endl;
		output <<"\t\t\tUdm::MetaDepository::RemoveDiagram(\"" << diagram.name() << "\");" << endl;
		output <<"\t\t};" <<endl;
		output <<"\t} __regUnUsed;" << endl << endl;

		output << "}" << endl <<"// END " << hname << ".cpp" <<endl;
}


// // --------------------------- CORBA CPP FORMAT Generation
// 
// void GenerateCORBACPP(const Uml::Diagram &diagram,  ostream &output, string fname, const Uml::Diagram& cross_dgr , const string& macro ) 
// {
// 		string hname = diagram.name();
// 		string::iterator i;
// 		for(i = hname.begin(); i != hname.end(); i++) {
// 			if(!isalnum(*i)) *i = '_';
// 		}
// 
// 		output << "// cpp source file " << fname << ".cpp generated from diagram " << (string)diagram.name() << endl;
// 		output << "// generated on " << GetTime().c_str() << endl << endl;
// 		output << "#include \""<< fname << ".h\"" <<endl;
// 		output << "#include \"UmlExt.h\"" <<endl << endl; 
// //		output << "#include \"UdmStatic.h\"" << endl << endl; 
// 
// 		output << "namespace " << hname << " {" << endl << endl;
// //	Uml::Diagram diagram;
// 		output << "\tUml::Diagram umldiagram;" << endl << endl;
// 
// //	Uml::Class A::meta;
// 		set<Uml::Class> classes = diagram.classes();
// 		set<Uml::Class>::iterator c;
// 		for(c = classes.begin(); c != classes.end(); c++ )
// 		{
// 			Uml::Class cl = *c;
// 			output << "\tUml::Class " << cl.name() << "::meta;" << endl;
// 
// 		}
// 
// //	Uml::Attribute A::meta_a;
// 		for(c = classes.begin(); c != classes.end(); c++ )	{
// 			Uml::Class cl = *c;
// //			Uml::Class tempcl(Uml::classByName(tempdiagram, cl.name()));
// 			set<Uml::Attribute> attributes = cl.attributes();
// /*			if(tempcl) {
// 				set<Uml::Attribute> tatts = tempcl.attributes();
// 				attributes.insert(tatts.begin(), tatts.end());
// 			}
// */			for( set<Uml::Attribute>::iterator i = attributes.begin(); i != attributes.end(); i++) {
// 				output << "\tUml::Attribute " << cl.name() << "::meta_" << (*i).name() << ";" << endl;
// 			}
// 		}
// 
// //	Uml::AssociationRole A::meta_dst;
// 		for(c = classes.begin(); c != classes.end(); c++ ) {
// 				Uml::Class cl = *c;
// 				set<Uml::AssociationRole> assocs = cl.associationRoles();
// 				for( set<Uml::AssociationRole>::iterator i = assocs.begin(); i != assocs.end(); i++) {
// 					Uml::Association pp = (*i).parent();
// 					Uml::Class aclass = pp.assocClass();
// 					string aname(MakeRoleName(Uml::theOther(*i)));
// 					if(!aclass) {
// 					output << "\tUml::AssociationRole " << cl.name() << "::meta_" << aname << ";" << endl;
// 					}
// 					else {
// 					output << "\tUml::AssociationRole " << cl.name() << "::meta_" << aname << ", " << cl.name() << "::meta_" << aname << "_rev;" << endl;
// 					}
// 			}
// //	Uml::AssociationRole A::meta_End1_end_;
// 
// 			Uml::Association assoc = cl.association();
// 			string s = assoc.meta.name();
// 			if(assoc) {
// 				set<Uml::AssociationRole> assocs = assoc.roles();
// 				for( set<Uml::AssociationRole>::iterator i = assocs.begin(); i != assocs.end(); i++) {
// 					string aname(MakeRoleName(*i));
// 					string oname(((Uml::Class)(*i).target()).name());
// 					if(aname.size()) {
// 					      output << "\tUml::AssociationRole " << cl.name() << "::meta_" << aname << "_end_;" << endl;
// 					}
// 				}
// 			}
// 		}
// 
// //	Uml::CompositionParentRole A::meta_parent;
// 		for(c = classes.begin(); c != classes.end(); c++ ) {
// 			Uml::Class cl = *c;
// 			set<Uml::CompositionChildRole> children = cl.childRoles();
// 			for( set<Uml::CompositionChildRole>::iterator i = children.begin(); i != children.end(); i++) {
// 				string aname(MakeRoleName(Uml::theOther(*i)));
// 				output << "\tUml::CompositionParentRole " << cl.name() << "::meta_" << aname << ";" << endl;
// 			}
// 		}
// 
// //	Uml::CompositionChildRole D::meta_as;
// 
// 		for(c = classes.begin(); c != classes.end(); c++ ) {
// 			Uml::Class cl = *c;
// 			set<Uml::CompositionParentRole> children = cl.parentRoles();
// 			for( set<Uml::CompositionParentRole>::iterator i = children.begin(); i != children.end(); i++) {
// 				string aname(MakeRoleName(Uml::theOther(*i)));
// 				output << "\tUml::CompositionChildRole " << cl.name() << "::meta_" << aname << ";" << endl;
// 			}
// 		}
// 
// //	void Initialize()
// //	{
// //		static bool first = true;
// //		if(!first) return;
// //		first = false;
// //		Uml::Initialize();
// //		ASSERT( umldiagram == Udm::null );
// //		umldiagram = Uml::CreateDiagram();
// 		output << "\tvoid Initialize()" << endl << "\t{" << endl;
// 		output << "\t\tstatic bool first = true;" << endl;
// 		output << "\t\tif(!first) return;" << endl;
// 		output << "\t\tfirst = false;" << endl;
// 		output << "\t\tUml::Initialize();" << endl << endl;
// 		output << "\t\tASSERT( umldiagram == Udm::null );" << endl;
// 		output << "\t\tumldiagram = Uml::CreateCORBADiagram();" << endl << endl;
// 		
// 
// //	A::meta = Uml::CreateClass();
// 		for(c = classes.begin(); c != classes.end(); c++ )
// 		{
// 			Uml::Class cl = *c;
// 			output << "\t\t" << cl.name() << "::meta = Uml::CreateCORBAClass();" << endl;
// 
// 		}
// 
// //	A::meta_a = Uml::CreateAttribute();
// 		for(c = classes.begin(); c != classes.end(); c++ )	{
// 			Uml::Class cl = *c;
// //			Uml::Class tempcl(Uml::classByName(tempdiagram, cl.name()));
// 			set<Uml::Attribute> attributes = cl.attributes();
// /*			if(tempcl) {
// 				set<Uml::Attribute> tatts = tempcl.attributes();
// 				attributes.insert(tatts.begin(), tatts.end());
// 			}
// */			for( set<Uml::Attribute>::iterator i = attributes.begin(); i != attributes.end(); i++) {
// 				output << "\t\t" << cl.name() << "::meta_" << (*i).name() << " = Uml::CreateCORBAAttribute();" << endl;
// 			}
// 		}
// 
// //	A::meta_dst = Uml::CreateAssociationRole();
// 		for(c = classes.begin(); c != classes.end(); c++ ) {
// 				Uml::Class cl = *c;
// 				set<Uml::AssociationRole> assocs = cl.associationRoles();
// 				for( set<Uml::AssociationRole>::iterator i = assocs.begin(); i != assocs.end(); i++) {
// 					string aname(MakeRoleName(Uml::theOther(*i)));
// 					output << "\t\t" << cl.name() << "::meta_" << aname << " = Uml::CreateCORBAAssociationRole();" << endl;
// 			}
// 		}
// 
// //	A::meta_parent = Uml::CreateCompositionParentRole();
// 		for(c = classes.begin(); c != classes.end(); c++ ) {
// 			Uml::Class cl = *c;
// 			set<Uml::CompositionChildRole> children = cl.childRoles();
// 			for( set<Uml::CompositionChildRole>::iterator i = children.begin(); i != children.end(); i++) {
// 				string aname(MakeRoleName(Uml::theOther(*i)));
// 				output << "\t\t" << cl.name() << "::meta_" << aname << " = Uml::CreateCORBACompositionParentRole();" << endl;
// 			}
// 		}
// 
// //	A::meta_as = Uml::CreateCompositionChildRole();
// 
// 		for(c = classes.begin(); c != classes.end(); c++ ) {
// 			Uml::Class cl = *c;
// 			set<Uml::CompositionParentRole> children = cl.parentRoles();
// 			for( set<Uml::CompositionParentRole>::iterator i = children.begin(); i != children.end(); i++) {
// 				string aname(MakeRoleName(Uml::theOther(*i)));
// 				output << "\t\t" << cl.name() << "::meta_" << aname << " = Uml::CreateCORBACompositionChildRole();" << endl;
// 			}
// 		}
// 
// 
// //		Uml::InitDiagram(diagram, "Sample");
// 		output << "\t\tUml::InitCORBADiagram(umldiagram, \"" << hname << "\");" << endl << endl;
// 
// //		Uml::InitClass(A::meta, diagram, "A", false);
// //	A::meta = Uml::CreateClass();
// 		for(c = classes.begin(); c != classes.end(); c++ )
// 		{
// 			Uml::Class cl = *c;
// 			string ster;
// 			if(((string)cl.stereotype()).length()) ster = string(", \"")+(string)cl.stereotype()+"\"";
// 
// 			string from;
// 			if(((string)cl.from()).length()) from= string(", \"")+(string)cl.from()+"\"";
// 			
// 			output << "\t\tUml::InitCORBAClass(" << cl.name() << "::meta, umldiagram, \"" <<
// 				cl.name() << "\", " << (cl.isAbstract() ? "true" : "false")<< ster << from << ");" << endl;
// 
// 		}
// 		output << "\t\t" << endl;
// 
// //		Uml::InitAttribute(A::meta_a, A::meta, "a", "String");
// 		for(c = classes.begin(); c != classes.end(); c++ )	{
// 			Uml::Class cl = *c;
// //			Uml::Class tempcl(Uml::classByName(tempdiagram, cl.name()));
// 			set<Uml::Attribute> attributes = cl.attributes();
// 
// 
// 			for( set<Uml::Attribute>::iterator i = attributes.begin(); i != attributes.end(); i++) 
// 			{
// 				//reproduce the defval string array
// 
// 				vector<string> dva = i->defvalue();
// 				output << "\t\tvector<string> " << cl.name() << "_" << (*i).name() << "_dva;" <<endl;;
// 
// 				vector<string>::const_iterator dva_ci = dva.begin();
// 	
// 				while (dva_ci != dva.end())
// 				{
// 					output << "\t\t" << cl.name() << "_" << (*i).name() << "_dva.push_back(\"" << *dva_ci++ << "\");" << endl; 
// 				};
// 
// 
// 				output << "\t\tUml::InitCORBAAttribute(" << cl.name() << "::meta_" << (*i).name() << ", " <<
// 					cl.name() << "::meta, \"" << (*i).name() << "\", \"" << (*i).type() << "\", " <<
// 					((*i).nonpersistent() ? "true" : "false") << ", " << (*i).min() << ", " << (*i).max() << ", " <<  
// 					((*i).ordered() ? "true" : "false") << ", \"" << (*i).visibility() <<
// 					"\", " << cl.name() << "_" << (*i).name() << "_dva" << ");" << endl;
// 			}
// 		}
// 		output << "\t\t" << endl;
// 
// 		{
// 		const set<Uml::Association> &asss = diagram.associations();
// 		set<Uml::Association>::const_iterator a;
// 		for(a = asss.begin(); a != asss.end(); a++ )	{
// 			set<Uml::AssociationRole> ar = a->roles();
// 			ASSERT(ar.size() == 2);
// //	{
// 			output << "\t\t{" << endl;
// //		Uml::Association ass = Uml::CreateAssociation();
// 			output << "\t\t\tUml::Association ass = Uml::CreateCORBAAssociation();" << endl;
// 
// //		Uml::InitAssociation(ass, diagram, "AtoB");
// // or : Uml::InitAssociationClass(ass, C::meta);
// 			Uml::Class aclass = a->assocClass();
// 			output << "\t\t\tUml::InitCORBAAssociation(ass , umldiagram, \"" << a->name() << "\");" << endl;
// 			if(aclass) {
// 				output << "\t\t\tUml::InitCORBAAssociationClass(ass, " << aclass.name() << "::meta);" << endl;
// 			}
// 
// //		Uml::InitAssociationRole(A::meta_dst, ass, "dst", 0, 1, B::meta);
// //								C:: meta_dst_end_ = B:meta_src_rev = A::meta_dst;
// 			set<Uml::AssociationRole> roles = a->roles();
// 			for( set<Uml::AssociationRole>::iterator i = roles.begin(); i != roles.end(); i++) {
// 					Uml::AssociationRole zz	= Uml::theOther(*i);
// 					string aname(MakeRoleName(zz));
// 					Uml::Class cl = i->target();
// 					output << "\t\t\tUml::InitCORBAAssociationRole(" << cl.name() << "::meta_" << aname 
// 						<< ", ass, \"" << aname << "\", " <<
// 						(zz.isNavigable() ? "true, " : "false, ") <<
// 						(zz.isPrimary() ? "true, " : "false, ") <<
// 						zz.min() << ", " << zz.max() << ", " << ((Uml::Class)zz.target()).name() << "::meta);" << endl;
// 					Uml::Class aclass = a->assocClass();
// 					if(aclass) {
// 						Uml::Class cl2 = zz.target();
// 						output << "\t\t\t" << aclass.name() << "::meta_" << aname << "_end_ = " <<
// 											cl2.name() << "::meta_" << MakeRoleName(*i) << "_rev = " << 
// 											cl.name() << "::meta_" << aname << ";" << endl;
// 					}
// 			}
// //	}
// 			output << "\t\t}" << endl;
// 		}
// 		}
// 
// 		{
// 		const set<Uml::Composition> &comps = diagram.compositions();
// 		set<Uml::Composition>::const_iterator c;
// 		for(c = comps.begin(); c != comps.end(); c++ )	{
// //	{
// 			output << "\t\t{" << endl;
// //		Uml::Composition comp = Uml::CreateComposition();
// 			output << "\t\t\tUml::Composition comp = Uml::CreateCORBAComposition();" << endl;
// 
// //		Uml::InitComposition(comp, diagram, "DtoA");
// 			output << "\t\t\tUml::InitCORBAComposition(comp , umldiagram, \"" << 
// 				c->name() << "\");" <<  endl;
// 			output << "\t\t" << endl;
// 
// 			Uml::CompositionParentRole x1 = (*c).parentRole();
// 			Uml::CompositionChildRole x2 = (*c).childRole();
// 
// //		Uml::InitCompositionParentRole(A::meta_parent, comp, "parent", D::meta);
// 			Uml::CompositionParentRole zzp = c->parentRole();
// 			Uml::CompositionChildRole zzc = c->childRole();
// 			string aname(MakeRoleName(zzp));
// 			output << "\t\t\tUml::InitCORBACompositionParentRole(" << ((Uml::Class)zzc.target()).name() << "::meta_" << aname << 
// 				", comp, \""<< aname << "\", " <<
// 				(zzp.isNavigable() ? "true, " : "false, ") <<
// 				((Uml::Class)zzp.target()).name() << "::meta);" << endl;
// 					
// //		Uml::InitCompositionChildRole(D::meta_as, comp, "as", true, 0, -1, A::meta); 
// 			string cname(MakeRoleName(zzc));
// 			output << "\t\t\tUml::InitCORBACompositionChildRole(" << ((Uml::Class)zzp.target()).name() << "::meta_" << cname << 
// 				", comp, \""<< MakeShortRoleName(zzc) << "\", " <<
// 					(zzc.isNavigable() ? "true, " : "false, ") <<
// 				zzc.min() << ", " << zzc.max() << ", " <<
// 				((Uml::Class)zzc.target()).name() << "::meta);" << endl;
// //	}
// 			output << "\t\t}" << endl;
// 		}
// 		}
// 
// 
// 
// //		Uml::AddInheritance(A::meta, C::meta);
// 		for(c = classes.begin(); c != classes.end(); c++ ) {
// 			Uml::Class cl = *c;
// 			set<Uml::Class> st = cl.subTypes();
// 			for( set<Uml::Class>::iterator i = st.begin(); i != st.end(); i++) {
// 				output << "\t\tUml::AddCORBAInheritance(" << cl.name() << "::meta, " << (*i).name() << "::meta);" << endl;
// 			}
// 		}			
// 		output << "\t\t" << endl << "\t}" << endl;
// //	Udm::UdmDiagram diagram = { &umldiagram, Initialize };
// 		output << "\tUdm::UdmDiagram diagram = { &umldiagram, Initialize };" << endl;
// 		output << "}" << endl <<"// END " << hname << ".cpp" <<endl;
// }



// -------------------------------------------------------------------------------------------------------------------------





void CheckClass(const Uml::Class & cl)
{
	set<string> unique_names;
	//reserved names
	unique_names.insert("parent");
	unique_names.insert("meta");
	unique_names.insert("archetype");
	unique_names.insert("derived");
	unique_names.insert("instances");
	unique_names.insert("desycnhed_atts");
	unique_names.insert((string)cl.name());
	unsigned int i;

	//attributes
	set<Uml::Attribute> attrs = Uml::AncestorAttributes(cl);
	set<Uml::Attribute>::iterator attrs_i = attrs.begin();
	while (attrs_i != attrs.end())
	{
		Uml::Attribute att = *attrs_i++;
		string att_name = att.name();

		i = unique_names.size();
		unique_names.insert(att_name);
		if (i == unique_names.size()) goto err_att;
		
		i = unique_names.size();
		unique_names.insert(string("meta_") + att_name);
		if (i == unique_names.size()) goto err_att;

		continue;
err_att:
		throw udm_exception("Reserved or duplicate attribute/role name: " + att_name);
	}


	//associationroles
	set<Uml::AssociationRole> assocs = cl.associationRoles();
	set<Uml::AssociationRole>::iterator assocs_i = assocs.begin();
	while (assocs_i != assocs.end())
	{
		
		Uml::AssociationRole ar = *assocs_i++;
		string aname(MakeRoleName(Uml::theOther(ar)));
		
		i = unique_names.size();
		unique_names.insert(aname);
		if (i == unique_names.size()) goto err_ar;
		
		i = unique_names.size();
		unique_names.insert(string("meta_") + aname);
		if (i == unique_names.size()) goto err_ar;

		continue;
err_ar:
		throw udm_exception("Reserved or duplicate attribute/role name: " + aname);

	}
	

	//compositionparentroles
	//check meta_child declarations
	set<Uml::CompositionParentRole> cprs = cl.parentRoles();
	set<Uml::CompositionParentRole>::iterator cprs_i = cprs.begin();
	set<Uml::Class> kinds;

	while (cprs_i != cprs.end())
	{
		Uml::CompositionParentRole cpr = *cprs_i++;
		string aname(MakeRoleName(Uml::theOther(cpr)));

		Uml::Class thischildkind = (Uml::Class)Uml::theOther(cpr).target();
		kinds.insert(thischildkind);
		string oname(thischildkind.name());

		i = unique_names.size();
		unique_names.insert(string("meta_") + aname);
		if (i == unique_names.size()) goto err_cpr;

		
		continue;
err_cpr:
		throw udm_exception("Reserved or duplicate attribute/role name: " + aname);
	}

	//check _kind_children stuff
	set<Uml::Class>::iterator kinds_i = kinds.begin();
	while (kinds_i != kinds.end())
	{
		i = unique_names.size();
		string kindname = (*kinds_i++).name();
		unique_names.insert(kindname + "_kind_children");
		if (i == unique_names.size()) goto err_kinds;
		continue;
err_kinds:
		throw udm_exception("Reserved or duplicate attribute/role name: " + kindname);
	}

	//check parent, meta_parent declarations 

	set<Uml::CompositionChildRole> ccrs = cl.childRoles();
	set<Uml::CompositionChildRole>::iterator ccrs_i = ccrs.begin();
	while (cprs_i != cprs.end())
	{
		Uml::CompositionChildRole ccr = *ccrs_i++;
		string aname(MakeRoleName(Uml::theOther(ccr)));

		i = unique_names.size();
		unique_names.insert(aname );
		if (i == unique_names.size()) goto err_ccr;

		i = unique_names.size();
		unique_names.insert(string("meta_") + aname );
		if (i == unique_names.size()) goto err_ccr;

		continue;
err_ccr:
		throw udm_exception("Reserved or duplicate attribute/role name: " + aname);
	}


};

void CheckDiagram(const Uml::Diagram & diagram)
{
	//check the UML diagram against duplicate & reserved role, attribute & classnames
	set<Uml::Class> cls = diagram.classes();
	set<Uml::Class>::iterator cls_i = cls.begin();

	set<string> unique_names;
	unique_names.insert(diagram.name());

	while (cls_i != cls.end())
	{
		Uml::Class cl = *cls_i++;
		unsigned int i = unique_names.size();
		unique_names.insert(cl.name());
		if (i == unique_names.size())
			throw udm_exception(string("Duplicate class name found: ") + (string)cl.name());

		CheckClass(cl);
	}
};
