/*  CHANGE LOG:
   
	1/30/2003 Aditya Agrawal 
	------------------------
	1. New FindAssocRole implementation, bug fixed and added capability
	2. Fixed bug in the POSTINCR implementation
	3. Fixed bug in the \t implementation
	4. Fixed bug in the DEFINE implementation
	5. Added new capability to <fieldspec> 
	   a) AssocEnd:<arolename>		returns the object at the arolename end from an association class
		
*/

#include <Uml.h>
UDM_USE_DOM

#ifdef _WIN32
UDM_USE_MGA
#endif

#include <iostream>
#include <fstream>
#include <cstdio>


#include <map>
#include <UmlExt.h>

#define PATLINELENGTH 500


class  FileGenerator {
private:
	ofstream fo;
	list<string *> outstrings;
	multimap<const string, string> vars;
	void outchannel_push(string &buf) {
			outstrings.push_front(&buf);
	}
	void outchannel_pop() {
			outstrings.pop_front();
	}

	void outc(string str) {
		if(!outstrings.empty()) {
			*outstrings.front() += str;
		}
		else if(fo) fo << str;
		else  {
				unsigned int pos = str.find_first_not_of(" \t");
				if(pos != string::npos) throw string("Nowhere to write generated text");
		}
	}
	void outc(char c) {
		if(!outstrings.empty()) {
			*outstrings.front() += c;
		}
		else if(fo) fo << c; 
		else {   // it it OK to silently ignore whitespace
				if(!isspace(c)) throw string("Nowhere to write generated text");
		}
	}
	int readarg(const char *pptr, string &outstr, char term);
	void evaluate_line(const string &line, const Udm::Object &context, bool escape);
	int evaluate_cmd(const string &ptr, const string &cmd, const Udm::Object &context);
public:
	void Define(string name, string value) {
		vars.insert(multimap<const string, string>::value_type(name, value));
	}
	FileGenerator() : fo() {
		;
	}
	int GenFile(const string &patfilename, const Udm::Object &context);
};



void eatss(const char *&ptr, string &outstr);
// on entry, ptr points to the opening '"'
// on exit, ptr points past the closing '"', but both '"' are omitted from the output.
void eatdq(const char *&ptr, string &outstr) {
		char c = 0;
		if(*ptr != '"') {    // non-quoted argument
			while(*ptr != ',' && *ptr != ')') {
			  if(*ptr == '$' && c != '\\') eatss(ptr, outstr);
			  else {
				c = *ptr++;
				if(c == '\0') throw 0;
				outstr += c;
			  }
			}
			return;
		}
		else {				   //  quoted argument
			ptr++;
			while(*ptr != '"' || c == '\\') {
				if(*ptr == '$' && c != '\\') eatss(ptr, outstr);
				else {
					c = *ptr++;
					if(c == '\0') throw 0;
					outstr += c;
				}
			}
			if(*ptr++ != '"') throw 0;
		}
}

// on entry ptr points to the $
// on exit, ptr points one char behind the closing ')'     both characters are printed
void eatss(const char *&ptr, string &outstr) {
	bool cmd = false;
	if(*ptr != '$') throw 0;
	outstr += *ptr++;
	if(*ptr == '(') {
		char c;
		do  {
			c = *ptr;
			if(c == '\0') throw 0;
			if(c == '"') {
				outstr += '"';
				eatdq(ptr, outstr);
				outstr += '"';
			}
			else if(c == '$') eatss(ptr, outstr);
			else outstr += *ptr++;
		} while(c != ')');
	}
	else {
		if(*ptr == '!') {
			cmd = true;
			outstr += *ptr++;
		}
		while(isalnum(*ptr) || *ptr == '_') {
			outstr += *ptr++;
		}
	}
	if(cmd && *ptr == '(') {
		char c;
		do  {
			c = *ptr;
			if(c == '\0') throw 0;
			if(c == '"') {
				outstr += '"';
				eatdq(ptr, outstr);
				outstr += '"';
			}
			else if(c == '$') eatss(ptr, outstr);
			else outstr += *ptr++;
		} while(c != ')');
	}
}

#define ANY_TERM ((char)0xFF)
// copy buffer until first unescaped ' " '
int FileGenerator::readarg(const char *pptr, string &outstr, char term) {
	const char *ptr = pptr;
	try {
		while(isspace(*ptr)) ptr++;
		eatdq(ptr, outstr);
		while(isspace(*ptr)) ptr++;
		if(*ptr++ != term && term != ANY_TERM) throw 0;
	}
	catch(int) {
		throw string("Error reading argument: ") + pptr;
	};
	return ptr - pptr;
}


const Uml::Class FindClass(Udm::Object obj, const string &name, bool survive = false) {
	set<Uml::Class> classes = ((Uml::Diagram)(obj.type().parent())).classes();
	for(set<Uml::Class>::iterator i = classes.begin(); i != classes.end(); i++) {
		if((string)(*i).name() == name) return *i;
	}
	if(!survive) throw string("Class " + name + " is not defined");
	return NULL;
}

const Uml::Attribute FindAttribute(Udm::Object &obj, const string &name, bool survive = false) {
		set<Uml::Class> cls = Uml::AncestorClasses(obj.type());
		for(set<Uml::Class>::iterator cl = cls.begin(); cl != cls.end(); cl++) {
			set<Uml::Attribute> attrs = cl->attributes();
			for(set<Uml::Attribute>::iterator i = attrs.begin(); i != attrs.end(); i++) {
				if((string)(*i).name() == name) return *i;
			}
	}
	if(!survive) throw string("Attribute " + name + " is not defined");
	return NULL;
}


const Uml::CompositionChildRole FindChildRole(Udm::Object obj, const string &name, bool survive = false) {
	set<Uml::Class> cls = Uml::AncestorClasses(obj.type());
	for(set<Uml::Class>::iterator cl = cls.begin(); cl != cls.end(); cl++) {
		set<Uml::CompositionChildRole> roles = (*cl).childRoles();
		for(set<Uml::CompositionChildRole>::iterator i = roles.begin(); i != roles.end(); i++) {
			if((string)(*i).name() == name) return *i;
		}
	}
	if(!survive) throw string("Child role " + name + " is not defined");
	return NULL;
}

const Uml::AssociationRole FindAssocRole(Udm::Object obj, const string &name, bool survive = false) {
	{	set<Uml::Class> cls = Uml::AncestorClasses(obj.type());
		for(set<Uml::Class>::iterator cl = cls.begin(); cl != cls.end(); cl++) {
			set<Uml::AssociationRole> roles = (*cl).associationRoles();
			for(set<Uml::AssociationRole>::iterator i = roles.begin(); i != roles.end(); i++) {
				if((string)(*i).name() == name) return *i;
			}
		}
	}
	{	set<Uml::Class> cls = Uml::AncestorClasses(obj.type());
		for(set<Uml::Class>::iterator cl = cls.begin(); cl != cls.end(); cl++) {
			Uml::Association ass = (*cl).association();
			set<Uml::AssociationRole> roles = ass.AssociationRole_kind_children();
			for(set<Uml::AssociationRole>::iterator i = roles.begin(); i != roles.end(); i++) {
				if((string)(*i).name() == name) return *i;
			}
		}
	}
	if(!survive) throw string("Association role " + name + " is not defined");
	return NULL;
}



const string GetAttribute(Udm::Object obj, const string &name, bool survive = false) {
	Uml::Attribute a = ::FindAttribute(obj,name);
	if(!a) throw "Undefined attribute referenced: " + name;
	string kk = a.type();
	char buf[100];
	if(kk == "String") return obj.__impl()->getStringAttr(a);
	else if(kk == "Integer") {
		sprintf(buf, "%d", (int)obj.__impl()->getIntegerAttr(a));
		return buf;
	}
	else if(kk == "Real") {
		sprintf(buf, "%g", (double)obj.__impl()->getRealAttr(a));
		return buf;
	}
	else if(kk == "Boolean") {
		return (bool)obj.__impl()->getBooleanAttr(a) ? "true" : "false";
	}
	else if(!survive) throw "Undefined attribute type: " + kk + " for attrname " + name;
	return "<undefined>";
}

int FileGenerator::evaluate_cmd(const string &pptr, const string &cmd, const Udm::Object &context) {
	const char *ptr = pptr.c_str();
	bool single;
	if((single = !strcmp(cmd.c_str(), "EVAL_WITH")) ||
	   !strcmp(cmd.c_str(), "EVAL_FORALL")) {
		string arg1; ptr += readarg(ptr, arg1, ',');
		string arg2; ptr += readarg(ptr, arg2, ')');
		string::size_type k = arg1.find(':');		
		string mode;
		if(k++ == string::npos) {
			if(FindChildRole(context, arg1, true)) mode = "ChildRole:";
			else if(FindClass(context, arg1, true)) mode = "ChildType:";
			else if(FindAssocRole(context, arg1, true)) mode = "AssocPeer:";
#ifdef _WIN32
			else if(!strcmpi(arg1.c_str(),"parent")) mode = "Parent:";
			else if(!strcmpi(arg1.c_str(),"type")) mode = "Type:";
#else
			else if(!strcasecmp(arg1.c_str(),"parent")) mode = "Parent:";
			else if(!strcasecmp(arg1.c_str(),"type")) mode = "Type:";

#endif
			else throw string("Cannot deduce operation for EVAL_xxx: ") + arg1;
		}
		else {
			mode = arg1.substr(0,k);
			arg1.erase(0,k);
		}
		vector<Udm::ObjectImpl *> v;	// this is where we collect the participants
		if(mode == "ChildRole:") {
			v = context.__impl()->getChildren(::FindChildRole(context, arg1), NULL);
		}
		else if(mode == "ChildType:") {
			v = context.__impl()->getChildren(NULL, ::FindClass(context, arg1));
		}
		else if(mode == "AssocPeer:") {
			v = context.__impl()->getAssociation(::FindAssocRole(context, arg1), Udm::TARGETFROMPEER);
		}
		else if(mode == "AssocClass:") {
			v = context.__impl()->getAssociation(::FindAssocRole(context, arg1), Udm::CLASSFROMTARGET);
		}
		else if(mode == "AssocEnd:") {
			v = context.__impl()->getAssociation(::FindAssocRole(context, arg1), Udm::TARGETFROMCLASS);
		}
		else if(mode == "Parent:") {
			v.push_back(context.__impl()->getParent(NULL));
		}
		else if(mode == "Type:") {
			v.push_back(context.__impl()->type().__impl()->clone());
		}
		else throw string("Unknown EVAL_xxx mode: ") + mode;

		if(single && v.size() != 1) throw string("EVAL_WITH results not a single object") + arg1;
		for( vector<Udm::ObjectImpl*>::iterator i = v.begin(); i != v.end() ; i++) {
				evaluate_line(arg2, *i, true);
		}
	}
	else if(!strcmp(cmd.c_str(), "DEFINE")) {
		string arg1; ptr += readarg(ptr, arg1, ',');
		string arg2; ptr += readarg(ptr, arg2, ')');
		string b;
		outchannel_push(b);
		evaluate_line(arg2, context, true);
		outchannel_pop();
		multimap<const string, string>::iterator variter = vars.find(arg1);
		if(variter != vars.end()) vars.erase(variter);
		Define(arg1, b);
	}
	else if(!strcmp(cmd.c_str(), "IFDEFAULT")) {
		string arg1; ptr += readarg(ptr, arg1, ',');
		string arg2; ptr += readarg(ptr, arg2, ')');
		string b = GetAttribute(context, arg1);
		if(b.empty()) {
			outchannel_push(b);
			evaluate_line(arg2, context, true);
			outchannel_pop();
		}
		else outc(b);
	}
	else if(!strcmp(cmd.c_str(), "IFEMPTY")) {
		string arg1; ptr += readarg(ptr, arg1, ',');
		string arg2; ptr += readarg(ptr, arg2, ')');
		string b;
		outchannel_push(b);
		evaluate_line(arg1, context, true);
		if(b.empty()) evaluate_line(arg2, context, true);
		outchannel_pop();
		outc(b);
	}
	else if(!strcmp(cmd.c_str(), "TO_FILE")) {
		string b;
		outchannel_push(b);
		string arg; ptr += readarg(ptr, arg, ')');
		evaluate_line(arg, context, true);
		outchannel_pop();
		if(fo.is_open()) fo.close();
		fo.clear();
		fo.open(b.c_str());
		if(!fo.good()) throw string("Cannot open output file")+b;
	}
	else if(!strcmp(cmd.c_str(), "POSTINCR")) {
		string arg; ptr += readarg(ptr, arg, ')');
		multimap<const string, string>::iterator variter = vars.find(arg);
		if(variter == vars.end()) throw string("POSTINCR: undefined var: ") + arg;
		string attrval = (*variter).second;
		outc(attrval);
		char buf[50]; sprintf(buf,"%d",atoi(attrval.c_str()) + 1);
		vars.erase(variter);
		Define(arg, buf);
	}
	else if(!strcmp(cmd.c_str(), "SEQ")) {

		do {
				string arg1; ptr += readarg(ptr, arg1, ANY_TERM);
				evaluate_line(arg1, context, true);
		} while(ptr[-1] == ',');
		if(ptr[-1] != ')') throw string("Unbalanced '(' ')' for SEQ");
//		string b;
//		outchannel_push(b);
//		evaluate_line(arg2, context, true);
//		outchannel_pop();
//		outc(b);
	}
	else if(!strcmp(cmd.c_str(), "GETPRINTFIELD")) {
		string arg; ptr += readarg(ptr, arg, ')');
		string b;
		outchannel_push(b);
		evaluate_line(arg, context, true);
		outchannel_pop();
		string code;
		if(!strcmp(b.c_str(),"int")) code = "%d";
		else if(!strcmp(b.c_str(),"long")) code = "%ld";
		else if(!strcmp(b.c_str(),"float")) code = "%f";
		else if(!strcmp(b.c_str(),"double")) code = "%f";
		else throw string("Cannot generate printf field for ")+b;
		outc(code);
	}
	else if(!strcmp(cmd.c_str(), "COMMENT")) {
		do {
				string arg1; ptr += readarg(ptr, arg1, ANY_TERM);
		} while(ptr[-1] == ',');
	}
	return ptr - pptr.c_str();
}


void FileGenerator::evaluate_line(const string &line, const Udm::Object &context, bool escape) {
			const char *ptr = line.c_str();
			bool in_esc = false;
			while( *ptr != '\0') {
				char c = *ptr++;
				if(escape && in_esc) {
						in_esc = false;
						if(c == 'n') outc('\n');
						else if(c == 't') outc('\t');
						else 		 outc(c);
				}
				else if(escape && c == '\\') in_esc = true;
				else if(c != '$') {
					outc(c);
				}
				else if(!escape && *ptr == '$') outc(*ptr++);
				else {								// it is a $macro
					bool endparen = false;
					bool command = false;
					if(*ptr == '!') {
						command = true;
						ptr++;
					}
					if(*ptr == '(') {
						endparen = true;
						ptr++;
					}
					char cmd[56];
					int add;
					if(sscanf(ptr, "%53[A-Za-z_0-9]%n", cmd, &add) != 1 ||
						add > 50 ||
						(endparen && ptr[add++] != ')') ||
						(command && ptr[add++] != '(')
												) throw string("Invalid $macro") ;
					ptr += add;
					if(command) {					// it is a $!macro
						ptr += evaluate_cmd(ptr, cmd, context);
					}
					else {							// it is a simple macro
						string attrval;
						multimap<const string, string>::iterator attiter = vars.find(cmd);
						if(attiter != vars.end()) attrval = attiter->second;
						else {
							attrval = GetAttribute(context, cmd);
						}
// ??						if(attrval == "") throw string("Attribute is empty: ")+cmd;
						outc(attrval);
					}
				}
			}
}			

int FileGenerator::GenFile(const string &patfilename, const Udm::Object &context) {
	int line = 0;
	ifstream fi(patfilename.c_str(), ios::in);
	try {
		if(!fi.good()) throw string("File not found: ");
		int rounds;
		for(line = 1;  ; line += rounds) {
			char buf[PATLINELENGTH+1];
			int pos = 0;
			rounds = 0;
			while(1) {
				fi.getline(buf+pos, sizeof(buf)-pos);
				if(fi.eof()) break;
				rounds++;
				pos = strlen(buf);
				if(pos >= PATLINELENGTH) throw(string("Too long line"));
				
				if(pos && buf[pos-1] == '\\') pos -= 1;   /* if line ends with '\' continue with next line */
				else { 
					buf[pos++] = '\n';
					buf[pos] = '\0';
					break;
				}
			} 
			if(fi.eof()) break;
			evaluate_line(buf, context, false);
		}
		if(!fo.is_open()) throw string("No output is generated");
	}
	catch(string e) {
		if(fo.is_open()) fo.close();
		cerr << "Error: " << e << "\nFile: " << patfilename << " line:" << line << "\n"; 
	}
	if(fo.is_open()) fo.close();
	return 0;
}

void dummy(void) {; }

int main(int argc, char **argv) {

	if(argc != 4) {
		cerr << "Usage: PatInterpreter <datalocator> <umldiagram> <patternfile>\n";
		cerr <<	"\tdatalocator:\t\tDOM:<domfilename> : open as DOM data\n";
		cerr << "\t\t\t\tGME:<projectname> : open as GME data \n";
		cerr << "\t\t\t\t<name_without_prefix> : open based on extension\n";
		cerr <<	"\tumldiagram: \t the UML-XML file that was used to create the data\n";
		cerr <<	"\tpatternfile: \t the pattern file\n";
		return -1;
	}
	try
	{
		
		Udm::SmartDataNetwork  nmeta(Uml::diagram);
		nmeta.OpenExisting(argv[2],"uml.dtd", Udm::CHANGES_LOST_DEFAULT);

		Uml::Diagram thedgr = Uml::Diagram::Cast(nmeta.GetRootObject());
		Udm::UdmDiagram datadiagram;
		datadiagram.dgr = &thedgr;
		datadiagram.init = dummy;

		Udm::SmartDataNetwork nw(datadiagram);

		try {
			nw.OpenExisting(argv[1],"", Udm::CHANGES_LOST_DEFAULT);
		
			FileGenerator().GenFile(argv[3],nw.GetRootObject());
		}
		catch(const udm_exception e)
		{
			cerr << "Inner Exception: " << e.what();
			return(-1);

		} 
	}
	catch(const udm_exception e)
	{
		cerr << "Exception: " << e.what();
		return(-1);

	} 
	return 0;
}
